/* This file is auto-generated, see gen-histogram-enum.py.  */


#ifndef mozilla_TelemetryHistogramEnums_h
#define mozilla_TelemetryHistogramEnums_h

#include "mozilla/TemplateLib.h"

namespace mozilla {
namespace Telemetry {

enum ID : uint32_t {
  A11Y_INSTANTIATED_FLAG,
  A11Y_CONSUMERS,
  A11Y_ISIMPLEDOM_USAGE_FLAG,
  A11Y_IATABLE_USAGE_FLAG,
  A11Y_UPDATE_TIME,
  ADDON_MANAGER_UPGRADE_UI_SHOWN,
  ADDON_SHIM_USAGE,
  ADDON_FORBIDDEN_CPOW_USAGE,
  BROWSER_SHIM_USAGE_BLOCKED,
  APPLICATION_REPUTATION_SHOULD_BLOCK,
  APPLICATION_REPUTATION_LOCAL,
  APPLICATION_REPUTATION_SERVER,
  APPLICATION_REPUTATION_SERVER_VERDICT,
  APPLICATION_REPUTATION_COUNT,
  APPLICATION_REPUTATION_REMOTE_LOOKUP_TIMEOUT,
  AUDIOSTREAM_FIRST_OPEN_MS,
  AUDIOSTREAM_LATER_OPEN_MS,
  AUDIOSTREAM_BACKEND_USED,
  BACKGROUNDFILESAVER_THREAD_COUNT,
  BATTERY_STATUS_COUNT,
  BLOCKLIST_SYNC_FILE_LOAD,
  CHECKERBOARD_DURATION,
  CHECKERBOARD_PEAK,
  CHECKERBOARD_POTENTIAL_DURATION,
  CHECKERBOARD_SEVERITY,
  COMPOSITE_TIME,
  COMPOSITE_FRAME_ROUNDTRIP_TIME,
  CONTENT_RESPONSE_DURATION,
  CYCLE_COLLECTOR,
  CYCLE_COLLECTOR_WORKER,
  CYCLE_COLLECTOR_FULL,
  CYCLE_COLLECTOR_MAX_PAUSE,
  CYCLE_COLLECTOR_FINISH_IGC,
  CYCLE_COLLECTOR_SYNC_SKIPPABLE,
  CYCLE_COLLECTOR_VISITED_REF_COUNTED,
  CYCLE_COLLECTOR_WORKER_VISITED_REF_COUNTED,
  CYCLE_COLLECTOR_VISITED_GCED,
  CYCLE_COLLECTOR_WORKER_VISITED_GCED,
  CYCLE_COLLECTOR_COLLECTED,
  CYCLE_COLLECTOR_WORKER_COLLECTED,
  CYCLE_COLLECTOR_NEED_GC,
  CYCLE_COLLECTOR_WORKER_NEED_GC,
  CYCLE_COLLECTOR_TIME_BETWEEN,
  CYCLE_COLLECTOR_OOM,
  CYCLE_COLLECTOR_WORKER_OOM,
  CYCLE_COLLECTOR_ASYNC_SNOW_WHITE_FREEING,
  DEFERRED_FINALIZE_ASYNC,
  DEVICE_RESET_REASON,
  FAMILY_SAFETY,
  FETCH_IS_MAINTHREAD,
  FORCED_DEVICE_RESET_REASON,
  FORGET_SKIPPABLE_MAX,
  FULLSCREEN_TRANSITION_BLACK_MS,
  FULLSCREEN_CHANGE_MS,
  GC_REASON_2,
  GC_IS_COMPARTMENTAL,
  GC_MS,
  GC_BUDGET_MS,
  GC_ANIMATION_MS,
  GC_MAX_PAUSE_MS,
  GC_MARK_MS,
  GC_SWEEP_MS,
  GC_COMPACT_MS,
  GC_MARK_ROOTS_MS,
  GC_MARK_GRAY_MS,
  GC_SLICE_MS,
  GC_SLOW_PHASE,
  GC_MMU_50,
  GC_RESET,
  GC_INCREMENTAL_DISABLED,
  GC_NON_INCREMENTAL,
  GC_SCC_SWEEP_TOTAL_MS,
  GC_SCC_SWEEP_MAX_PAUSE_MS,
  GC_MINOR_REASON,
  GC_MINOR_REASON_LONG,
  GC_MINOR_US,
  GEOLOCATION_ACCURACY_EXPONENTIAL,
  GEOLOCATION_ERROR,
  GEOLOCATION_GETCURRENTPOSITION_SECURE_ORIGIN,
  GEOLOCATION_REQUEST_GRANTED,
  GEOLOCATION_WATCHPOSITION_SECURE_ORIGIN,
  GEOLOCATION_WIN8_SOURCE_IS_MLS,
  GEOLOCATION_OSX_SOURCE_IS_MLS,
  GEOLOCATION_GETCURRENTPOSITION_VISIBLE,
  GEOLOCATION_WATCHPOSITION_VISIBLE,
  JS_DEPRECATED_LANGUAGE_EXTENSIONS_IN_CONTENT,
  JS_DEPRECATED_LANGUAGE_EXTENSIONS_IN_ADDONS,
  XUL_CACHE_DISABLED,
  MEMORY_RESIDENT_FAST,
  MEMORY_TOTAL,
  MEMORY_UNIQUE,
  MEMORY_VSIZE,
  MEMORY_VSIZE_MAX_CONTIGUOUS,
  MEMORY_JS_COMPARTMENTS_SYSTEM,
  MEMORY_JS_COMPARTMENTS_USER,
  MEMORY_JS_GC_HEAP,
  MEMORY_STORAGE_SQLITE,
  MEMORY_IMAGES_CONTENT_USED_UNCOMPRESSED,
  MEMORY_HEAP_ALLOCATED,
  MEMORY_HEAP_COMMITTED_UNUSED,
  MEMORY_HEAP_OVERHEAD_FRACTION,
  GHOST_WINDOWS,
  MEMORY_FREE_PURGED_PAGES_MS,
#if defined(XP_WIN)
  LOW_MEMORY_EVENTS_VIRTUAL,
#endif
#if defined(XP_WIN)
  LOW_MEMORY_EVENTS_PHYSICAL,
#endif
#if defined(XP_WIN)
  LOW_MEMORY_EVENTS_COMMIT_SPACE,
#endif
#if defined(XP_UNIX)
  PAGE_FAULTS_HARD,
#endif
  FONTLIST_INITOTHERFAMILYNAMES,
  FONTLIST_INITFACENAMELISTS,
#if defined(XP_WIN)
  DWRITEFONT_DELAYEDINITFONTLIST_TOTAL,
#endif
#if defined(XP_WIN)
  DWRITEFONT_DELAYEDINITFONTLIST_COUNT,
#endif
#if defined(XP_WIN)
  DWRITEFONT_DELAYEDINITFONTLIST_COLLECT,
#endif
#if defined(XP_WIN)
  DWRITEFONT_INIT_PROBLEM,
#endif
#if defined(XP_WIN)
  GDI_INITFONTLIST_TOTAL,
#endif
#if defined(XP_DARWIN)
  MAC_INITFONTLIST_TOTAL,
#endif
  SYSTEM_FONT_FALLBACK,
  SYSTEM_FONT_FALLBACK_FIRST,
  SYSTEM_FONT_FALLBACK_SCRIPT,
  GRADIENT_DURATION,
  GRADIENT_RETENTION_TIME,
  STARTUP_CACHE_AGE_HOURS,
  STARTUP_CACHE_INVALID,
  WORD_CACHE_HITS_CONTENT,
  WORD_CACHE_HITS_CHROME,
  WORD_CACHE_MISSES_CONTENT,
  WORD_CACHE_MISSES_CHROME,
  FONT_CACHE_HIT,
  BAD_FALLBACK_FONT,
  SHUTDOWN_OK,
  IMAGE_DECODE_LATENCY_US,
  IMAGE_DECODE_TIME,
  IMAGE_DECODE_ON_DRAW_LATENCY,
  IMAGE_DECODE_CHUNKS,
  IMAGE_DECODE_COUNT,
  IMAGE_DECODE_SPEED_JPEG,
  IMAGE_DECODE_SPEED_GIF,
  IMAGE_DECODE_SPEED_PNG,
  IMAGE_OPTIMIZE_TO_SINGLE_COLOR_USED,
  IMAGE_OPTIMIZE_TO_565_USED,
  CANVAS_2D_USED,
  CANVAS_WEBGL_ACCL_FAILURE_ID,
  CANVAS_WEBGL_FAILURE_ID,
  CANVAS_WEBGL_SUCCESS,
  CANVAS_WEBGL_USED,
  CANVAS_WEBGL2_SUCCESS,
  TOTAL_CONTENT_PAGE_LOAD_TIME,
  HTTP_SUBITEM_OPEN_LATENCY_TIME,
  HTTP_SUBITEM_FIRST_BYTE_LATENCY_TIME,
  HTTP_REQUEST_PER_PAGE,
  HTTP_REQUEST_PER_PAGE_FROM_CACHE,
  HTTP_REQUEST_PER_CONN,
  HTTP_KBREAD_PER_CONN,
  HTTP_PAGE_DNS_ISSUE_TIME,
  HTTP_PAGE_DNS_LOOKUP_TIME,
  HTTP_PAGE_TCP_CONNECTION,
  HTTP_PAGE_OPEN_TO_FIRST_SENT,
  HTTP_PAGE_FIRST_SENT_TO_LAST_RECEIVED,
  HTTP_PAGE_OPEN_TO_FIRST_RECEIVED,
  HTTP_PAGE_OPEN_TO_FIRST_FROM_CACHE,
  HTTP_PAGE_OPEN_TO_FIRST_FROM_CACHE_V2,
  HTTP_PAGE_CACHE_READ_TIME,
  HTTP_PAGE_CACHE_READ_TIME_V2,
  HTTP_PAGE_REVALIDATION,
  HTTP_PAGE_COMPLETE_LOAD,
  HTTP_PAGE_COMPLETE_LOAD_V2,
  HTTP_PAGE_COMPLETE_LOAD_CACHED,
  HTTP_PAGE_COMPLETE_LOAD_CACHED_V2,
  HTTP_PAGE_COMPLETE_LOAD_NET,
  HTTP_PAGE_COMPLETE_LOAD_NET_V2,
  HTTP_SUB_DNS_ISSUE_TIME,
  HTTP_SUB_DNS_LOOKUP_TIME,
  HTTP_SUB_TCP_CONNECTION,
  HTTP_SUB_OPEN_TO_FIRST_SENT,
  HTTP_SUB_FIRST_SENT_TO_LAST_RECEIVED,
  HTTP_SUB_OPEN_TO_FIRST_RECEIVED,
  HTTP_SUB_OPEN_TO_FIRST_FROM_CACHE,
  HTTP_SUB_OPEN_TO_FIRST_FROM_CACHE_V2,
  HTTP_SUB_CACHE_READ_TIME,
  HTTP_SUB_CACHE_READ_TIME_V2,
  HTTP_SUB_REVALIDATION,
  HTTP_SUB_COMPLETE_LOAD,
  HTTP_SUB_COMPLETE_LOAD_V2,
  HTTP_SUB_COMPLETE_LOAD_CACHED,
  HTTP_SUB_COMPLETE_LOAD_CACHED_V2,
  HTTP_SUB_COMPLETE_LOAD_NET,
  HTTP_SUB_COMPLETE_LOAD_NET_V2,
  HTTP_PROXY_TYPE,
  HTTP_TRANSACTION_IS_SSL,
  HTTP_PAGELOAD_IS_SSL,
  HTTP_TRANSACTION_USE_ALTSVC,
  HTTP_TRANSACTION_USE_ALTSVC_OE,
  HTTP_SCHEME_UPGRADE,
  HTTP_RESPONSE_STATUS_CODE,
  HTTP_AUTH_DIALOG_STATS,
  HTTP_AUTH_TYPE_STATS,
  SSL_HANDSHAKE_VERSION,
  SSL_TIME_UNTIL_READY,
  SSL_TIME_UNTIL_HANDSHAKE_FINISHED,
  SSL_BYTES_BEFORE_CERT_CALLBACK,
  SSL_NPN_TYPE,
  SSL_RESUMED_SESSION,
  CERT_VALIDATION_HTTP_REQUEST_RESULT,
  CERT_VALIDATION_HTTP_REQUEST_CANCELED_TIME,
  CERT_VALIDATION_HTTP_REQUEST_SUCCEEDED_TIME,
  CERT_VALIDATION_HTTP_REQUEST_FAILED_TIME,
  SSL_KEY_EXCHANGE_ALGORITHM_FULL,
  SSL_KEY_EXCHANGE_ALGORITHM_RESUMED,
  SSL_OBSERVED_END_ENTITY_CERTIFICATE_LIFETIME,
  KEYGEN_GENERATED_KEY_TYPE,
  WEBSOCKETS_HANDSHAKE_TYPE,
  SPDY_VERSION2,
  HTTP_RESPONSE_VERSION,
  HTTP_09_INFO,
  SPDY_PARALLEL_STREAMS,
  SPDY_REQUEST_PER_CONN,
  SPDY_SERVER_INITIATED_STREAMS,
  SPDY_CHUNK_RECVD,
  SPDY_SYN_SIZE,
  SPDY_SYN_RATIO,
  SPDY_SYN_REPLY_SIZE,
  SPDY_SYN_REPLY_RATIO,
  SPDY_NPN_CONNECT,
  SPDY_NPN_JOIN,
  SPDY_KBREAD_PER_CONN,
  SPDY_SETTINGS_UL_BW,
  SPDY_SETTINGS_DL_BW,
  SPDY_SETTINGS_RTT,
  SPDY_SETTINGS_MAX_STREAMS,
  SPDY_SETTINGS_CWND,
  SPDY_SETTINGS_RETRANS,
  SPDY_SETTINGS_IW,
  SPDY_GOAWAY_LOCAL,
  SPDY_GOAWAY_PEER,
  HTTP_CONNECTION_ENTRY_CACHE_HIT_1,
  HTTP_CACHE_DISPOSITION_2,
  HTTP_CACHE_DISPOSITION_2_V2,
  HTTP_DISK_CACHE_DISPOSITION_2,
  HTTP_CACHE_MISS_HALFLIFE_EXPERIMENT_2,
  HTTP_CACHE_ENTRY_RELOAD_TIME,
  HTTP_CACHE_ENTRY_ALIVE_TIME,
  HTTP_CACHE_ENTRY_REUSE_COUNT,
  HTTP_MEMORY_CACHE_DISPOSITION_2,
  HTTP_OFFLINE_CACHE_DISPOSITION_2,
  HTTP_OFFLINE_CACHE_DOCUMENT_LOAD,
  HTTP_CACHE_IO_QUEUE_OPEN_PRIORITY,
  HTTP_CACHE_IO_QUEUE_READ_PRIORITY,
  HTTP_CACHE_IO_QUEUE_OPEN,
  HTTP_CACHE_IO_QUEUE_READ,
  HTTP_CACHE_IO_QUEUE_MANAGEMENT,
  HTTP_CACHE_IO_QUEUE_WRITE,
  HTTP_CACHE_IO_QUEUE_INDEX,
  HTTP_CACHE_IO_QUEUE_EVICT,
  CACHE_DEVICE_SEARCH_2,
  CACHE_MEMORY_SEARCH_2,
  CACHE_DISK_SEARCH_2,
  CACHE_OFFLINE_SEARCH_2,
  TRANSACTION_WAIT_TIME_HTTP,
  TRANSACTION_WAIT_TIME_HTTP_PIPELINES,
  TRANSACTION_WAIT_TIME_SPDY,
  HTTP_SAW_QUIC_ALT_PROTOCOL,
  HTTP_CONTENT_ENCODING,
  HTTP_DISK_CACHE_OVERHEAD,
  CACHE_LM_INCONSISTENT,
  CACHE_SERVICE_LOCK_WAIT_2,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_2,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSSETDISKSMARTSIZECALLBACK_NOTIFY,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSPROCESSREQUESTEVENT_RUN,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSOUTPUTSTREAMWRAPPER_LAZYINIT,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSOUTPUTSTREAMWRAPPER_CLOSEINTERNAL,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSOUTPUTSTREAMWRAPPER_RELEASE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCOMPRESSOUTPUTSTREAMWRAPPER_RELEASE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSINPUTSTREAMWRAPPER_LAZYINIT,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSINPUTSTREAMWRAPPER_CLOSEINTERNAL,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSINPUTSTREAMWRAPPER_RELEASE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSDECOMPRESSINPUTSTREAMWRAPPER_RELEASE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_SHUTDOWN,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_SETOFFLINECACHEENABLED,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_SETOFFLINECACHECAPACITY,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_SETMEMORYCACHE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_SETDISKSMARTSIZE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_SETDISKCACHEMAXENTRYSIZE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_SETMEMORYCACHEMAXENTRYSIZE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_SETDISKCACHEENABLED,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_SETDISKCACHECAPACITY,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_OPENCACHEENTRY,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_ONPROFILESHUTDOWN,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_ONPROFILECHANGED,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_ISSTORAGEENABLEDFORPOLICY,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_GETCACHEIOTARGET,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_EVICTENTRIESFORCLIENT,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_DISKDEVICEHEAPSIZE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_CLOSEALLSTREAMS,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_DOOM,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_SETPREDICTEDDATASIZE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETDATASIZE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETSTORAGEDATASIZE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_REQUESTDATASIZECHANGE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_SETDATASIZE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_OPENINPUTSTREAM,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_OPENOUTPUTSTREAM,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETCACHEELEMENT,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_SETCACHEELEMENT,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETSTORAGEPOLICY,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_SETSTORAGEPOLICY,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETFILE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETSECURITYINFO,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_SETSECURITYINFO,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_DOOMANDFAILPENDINGREQUESTS,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_MARKVALID,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_CLOSE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETMETADATAELEMENT,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_SETMETADATAELEMENT,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_VISITMETADATA,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_SETEXPIRATIONTIME,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_ISSTREAMBASED,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETLASTMODIFIED,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETEXPIRATIONTIME,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETKEY,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETFETCHCOUNT,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETDEVICEID,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_PROCESSREQUEST,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHESERVICE_VISITENTRIES,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETPREDICTEDDATASIZE,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETLASTFETCHED,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSCACHEENTRYDESCRIPTOR_GETCLIENTID,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSBLOCKONCACHETHREADEVENT_RUN,
  CACHE_SERVICE_LOCK_WAIT_MAINTHREAD_NSASYNCDOOMEVENT_RUN,
  DNT_USAGE,
  DNS_LOOKUP_METHOD2,
  DNS_CLEANUP_AGE,
  DNS_LOOKUP_TIME,
  DNS_RENEWAL_TIME,
  DNS_RENEWAL_TIME_FOR_TTL,
  DNS_FAILED_LOOKUP_TIME,
  DNS_BLACKLIST_COUNT,
  REFRESH_DRIVER_TICK,
  PAINT_BUILD_DISPLAYLIST_TIME,
  PAINT_RASTERIZE_TIME,
  PREDICTOR_PREDICT_ATTEMPTS,
  PREDICTOR_LEARN_ATTEMPTS,
  PREDICTOR_PREDICT_FULL_QUEUE,
  PREDICTOR_LEARN_FULL_QUEUE,
  PREDICTOR_WAIT_TIME,
  PREDICTOR_PREDICT_WORK_TIME,
  PREDICTOR_LEARN_WORK_TIME,
  PREDICTOR_TOTAL_PREDICTIONS,
  PREDICTOR_TOTAL_PREFETCHES,
  PREDICTOR_TOTAL_PREFETCHES_USED,
  PREDICTOR_PREFETCH_TIME,
  PREDICTOR_TOTAL_PRECONNECTS,
  PREDICTOR_TOTAL_PRECONNECTS_CREATED,
  PREDICTOR_TOTAL_PRECONNECTS_USED,
  PREDICTOR_TOTAL_PRECONNECTS_UNUSED,
  PREDICTOR_TOTAL_PRERESOLVES,
  PREDICTOR_PREDICTIONS_CALCULATED,
  PREDICTOR_GLOBAL_DEGRADATION,
  PREDICTOR_SUBRESOURCE_DEGRADATION,
  PREDICTOR_BASE_CONFIDENCE,
  PREDICTOR_CONFIDENCE,
  PREDICTOR_PREDICT_TIME_TO_ACTION,
  PREDICTOR_PREDICT_TIME_TO_INACTION,
  HTTPCONNMGR_TOTAL_SPECULATIVE_CONN,
  HTTPCONNMGR_USED_SPECULATIVE_CONN,
  HTTPCONNMGR_UNUSED_SPECULATIVE_CONN,
  TAP_TO_LOAD_IMAGE_SIZE,
  STS_POLL_AND_EVENTS_CYCLE,
  STS_NUMBER_OF_PENDING_EVENTS,
  STS_POLL_CYCLE,
  STS_POLL_AND_EVENT_THE_LAST_CYCLE,
  STS_NUMBER_OF_PENDING_EVENTS_IN_THE_LAST_CYCLE,
  STS_NUMBER_OF_ONSOCKETREADY_CALLS,
  STS_POLL_BLOCK_TIME,
  PRCONNECT_BLOCKING_TIME_NORMAL,
  PRCONNECT_BLOCKING_TIME_SHUTDOWN,
  PRCONNECT_BLOCKING_TIME_CONNECTIVITY_CHANGE,
  PRCONNECT_BLOCKING_TIME_LINK_CHANGE,
  PRCONNECT_BLOCKING_TIME_OFFLINE,
  PRCONNECT_FAIL_BLOCKING_TIME_NORMAL,
  PRCONNECT_FAIL_BLOCKING_TIME_SHUTDOWN,
  PRCONNECT_FAIL_BLOCKING_TIME_CONNECTIVITY_CHANGE,
  PRCONNECT_FAIL_BLOCKING_TIME_LINK_CHANGE,
  PRCONNECT_FAIL_BLOCKING_TIME_OFFLINE,
  PRCONNECTCONTINUE_BLOCKING_TIME_NORMAL,
  PRCONNECTCONTINUE_BLOCKING_TIME_SHUTDOWN,
  PRCONNECTCONTINUE_BLOCKING_TIME_CONNECTIVITY_CHANGE,
  PRCONNECTCONTINUE_BLOCKING_TIME_LINK_CHANGE,
  PRCONNECTCONTINUE_BLOCKING_TIME_OFFLINE,
  PRCLOSE_TCP_BLOCKING_TIME_NORMAL,
  PRCLOSE_TCP_BLOCKING_TIME_SHUTDOWN,
  PRCLOSE_TCP_BLOCKING_TIME_CONNECTIVITY_CHANGE,
  PRCLOSE_TCP_BLOCKING_TIME_LINK_CHANGE,
  PRCLOSE_TCP_BLOCKING_TIME_OFFLINE,
  PRCLOSE_UDP_BLOCKING_TIME_NORMAL,
  PRCLOSE_UDP_BLOCKING_TIME_SHUTDOWN,
  PRCLOSE_UDP_BLOCKING_TIME_CONNECTIVITY_CHANGE,
  PRCLOSE_UDP_BLOCKING_TIME_LINK_CHANGE,
  PRCLOSE_UDP_BLOCKING_TIME_OFFLINE,
  IPV4_AND_IPV6_ADDRESS_CONNECTIVITY,
  NETWORK_SESSION_AT_900FD,
  NETWORK_PROBE_MAXCOUNT,
  FIND_PLUGINS,
  CHECK_JAVA_ENABLED,
  PLUGIN_HANG_UI_USER_RESPONSE,
  PLUGIN_HANG_UI_DONT_ASK,
  PLUGIN_HANG_UI_RESPONSE_TIME,
  PLUGIN_HANG_TIME,
  PLUGIN_LOAD_METADATA,
  PLUGIN_SHUTDOWN_MS,
  PLUGIN_CALLED_DIRECTLY,
  FLASH_PLUGIN_STATES,
  FLASH_PLUGIN_AREA,
  FLASH_PLUGIN_WIDTH,
  FLASH_PLUGIN_HEIGHT,
  FLASH_PLUGIN_INSTANCES_ON_PAGE,
  MOZ_SQLITE_OPEN_MS,
  MOZ_SQLITE_OPEN_MAIN_THREAD_MS,
  MOZ_SQLITE_TRUNCATE_MS,
  MOZ_SQLITE_TRUNCATE_MAIN_THREAD_MS,
  MOZ_SQLITE_OTHER_READ_MS,
  MOZ_SQLITE_OTHER_READ_MAIN_THREAD_MS,
  MOZ_SQLITE_PLACES_READ_MS,
  MOZ_SQLITE_PLACES_READ_MAIN_THREAD_MS,
  MOZ_SQLITE_COOKIES_OPEN_READAHEAD_MS,
  MOZ_SQLITE_COOKIES_READ_MS,
  MOZ_SQLITE_COOKIES_READ_MAIN_THREAD_MS,
  MOZ_SQLITE_WEBAPPS_READ_MS,
  MOZ_SQLITE_WEBAPPS_READ_MAIN_THREAD_MS,
  MOZ_SQLITE_OTHER_WRITE_MS,
  MOZ_SQLITE_OTHER_WRITE_MAIN_THREAD_MS,
  MOZ_SQLITE_PLACES_WRITE_MS,
  MOZ_SQLITE_PLACES_WRITE_MAIN_THREAD_MS,
  MOZ_SQLITE_COOKIES_WRITE_MS,
  MOZ_SQLITE_COOKIES_WRITE_MAIN_THREAD_MS,
  MOZ_SQLITE_WEBAPPS_WRITE_MS,
  MOZ_SQLITE_WEBAPPS_WRITE_MAIN_THREAD_MS,
  MOZ_SQLITE_OTHER_SYNC_MS,
  MOZ_SQLITE_OTHER_SYNC_MAIN_THREAD_MS,
  MOZ_SQLITE_PLACES_SYNC_MS,
  MOZ_SQLITE_PLACES_SYNC_MAIN_THREAD_MS,
  MOZ_SQLITE_COOKIES_SYNC_MS,
  MOZ_SQLITE_COOKIES_SYNC_MAIN_THREAD_MS,
  MOZ_SQLITE_WEBAPPS_SYNC_MS,
  MOZ_SQLITE_WEBAPPS_SYNC_MAIN_THREAD_MS,
  MOZ_SQLITE_OTHER_READ_B,
  MOZ_SQLITE_PLACES_READ_B,
  MOZ_SQLITE_COOKIES_READ_B,
  MOZ_SQLITE_WEBAPPS_READ_B,
  MOZ_SQLITE_PLACES_WRITE_B,
  MOZ_SQLITE_COOKIES_WRITE_B,
  MOZ_SQLITE_WEBAPPS_WRITE_B,
  MOZ_SQLITE_OTHER_WRITE_B,
  MOZ_STORAGE_ASYNC_REQUESTS_MS,
  MOZ_STORAGE_ASYNC_REQUESTS_SUCCESS,
  STARTUP_MEASUREMENT_ERRORS,
  NETWORK_DISK_CACHE_OPEN,
  NETWORK_DISK_CACHE_TRASHRENAME,
  NETWORK_DISK_CACHE_DELETEDIR,
  NETWORK_DISK_CACHE_DELETEDIR_SHUTDOWN,
  NETWORK_DISK_CACHE_SHUTDOWN,
  NETWORK_DISK_CACHE_SHUTDOWN_V2,
  NETWORK_DISK_CACHE_SHUTDOWN_CLEAR_PRIVATE,
  NETWORK_DISK_CACHE2_SHUTDOWN_CLEAR_PRIVATE,
  NETWORK_ID,
  IDLE_NOTIFY_IDLE_MS,
  URLCLASSIFIER_LOOKUP_TIME,
  URLCLASSIFIER_CL_CHECK_TIME,
  URLCLASSIFIER_CL_UPDATE_TIME,
  URLCLASSIFIER_PS_FILELOAD_TIME,
  URLCLASSIFIER_PS_FALLOCATE_TIME,
  URLCLASSIFIER_PS_CONSTRUCT_TIME,
  URLCLASSIFIER_LC_PREFIXES,
  URLCLASSIFIER_LC_COMPLETIONS,
  URLCLASSIFIER_PS_FAILURE,
  URLCLASSIFIER_UPDATE_REMOTE_STATUS,
  URLCLASSIFIER_COMPLETE_REMOTE_STATUS,
  URLCLASSIFIER_COMPLETE_TIMEOUT,
  CSP_DOCUMENTS_COUNT,
  CSP_UNSAFE_INLINE_DOCUMENTS_COUNT,
  CSP_UNSAFE_EVAL_DOCUMENTS_COUNT,
  PLACES_PAGES_COUNT,
  PLACES_MOST_RECENT_EXPIRED_VISIT_DAYS,
  PLACES_BOOKMARKS_COUNT,
  PLACES_TAGS_COUNT,
  PLACES_KEYWORDS_COUNT,
  PLACES_BACKUPS_DAYSFROMLAST,
  PLACES_BACKUPS_BOOKMARKSTREE_MS,
  PLACES_BACKUPS_TOJSON_MS,
  PLACES_EXPORT_TOHTML_MS,
  PLACES_FAVICON_ICO_SIZES,
  PLACES_FAVICON_PNG_SIZES,
  PLACES_FAVICON_GIF_SIZES,
  PLACES_FAVICON_JPEG_SIZES,
  PLACES_FAVICON_BMP_SIZES,
  PLACES_FAVICON_SVG_SIZES,
  PLACES_FAVICON_OTHER_SIZES,
  LINK_ICON_SIZES_ATTR_USAGE,
  LINK_ICON_SIZES_ATTR_DIMENSION,
#if defined(ANDROID)
  FENNEC_DISTRIBUTION_REFERRER_INVALID,
#endif
#if defined(ANDROID)
  FENNEC_DISTRIBUTION_CODE_CATEGORY,
#endif
#if defined(ANDROID)
  FENNEC_DISTRIBUTION_DOWNLOAD_TIME_MS,
#endif
  FENNEC_BOOKMARKS_COUNT,
#if defined(ANDROID)
  FENNEC_READING_LIST_COUNT,
#endif
  FENNEC_READER_VIEW_CACHE_SIZE,
  PLACES_SORTED_BOOKMARKS_PERC,
  PLACES_TAGGED_BOOKMARKS_PERC,
  PLACES_DATABASE_FILESIZE_MB,
  PLACES_DATABASE_PAGESIZE_B,
  PLACES_DATABASE_SIZE_PER_PAGE_B,
  PLACES_EXPIRATION_STEPS_TO_CLEAN2,
  PLACES_AUTOCOMPLETE_1ST_RESULT_TIME_MS,
  PLACES_AUTOCOMPLETE_6_FIRST_RESULTS_TIME_MS,
  HISTORY_LASTVISITED_TREE_QUERY_TIME_MS,
  PLACES_HISTORY_LIBRARY_SEARCH_TIME_MS,
  PLACES_AUTOCOMPLETE_URLINLINE_DOMAIN_QUERY_TIME_MS,
  PLACES_IDLE_FRECENCY_DECAY_TIME_MS,
  PLACES_IDLE_MAINTENANCE_TIME_MS,
  PLACES_ANNOS_BOOKMARKS_COUNT,
  PLACES_ANNOS_PAGES_COUNT,
  PLACES_MAINTENANCE_DAYSFROMLAST,
  UPDATE_CHECK_NO_UPDATE_EXTERNAL,
  UPDATE_CHECK_NO_UPDATE_NOTIFY,
  UPDATE_CHECK_CODE_EXTERNAL,
  UPDATE_CHECK_CODE_NOTIFY,
  UPDATE_CHECK_EXTENDED_ERROR_EXTERNAL,
  UPDATE_CHECK_EXTENDED_ERROR_NOTIFY,
  UPDATE_INVALID_LASTUPDATETIME_EXTERNAL,
  UPDATE_INVALID_LASTUPDATETIME_NOTIFY,
  UPDATE_LAST_NOTIFY_INTERVAL_DAYS_EXTERNAL,
  UPDATE_LAST_NOTIFY_INTERVAL_DAYS_NOTIFY,
  UPDATE_PING_COUNT_EXTERNAL,
  UPDATE_PING_COUNT_NOTIFY,
  UPDATE_SERVICE_INSTALLED_EXTERNAL,
  UPDATE_SERVICE_INSTALLED_NOTIFY,
  UPDATE_SERVICE_MANUALLY_UNINSTALLED_EXTERNAL,
  UPDATE_SERVICE_MANUALLY_UNINSTALLED_NOTIFY,
  UPDATE_UNABLE_TO_APPLY_EXTERNAL,
  UPDATE_UNABLE_TO_APPLY_NOTIFY,
  UPDATE_CANNOT_STAGE_EXTERNAL,
  UPDATE_CANNOT_STAGE_NOTIFY,
  UPDATE_HAS_PREF_URL_OVERRIDE_EXTERNAL,
  UPDATE_HAS_PREF_URL_OVERRIDE_NOTIFY,
  UPDATE_PREF_UPDATE_CANCELATIONS_EXTERNAL,
  UPDATE_PREF_UPDATE_CANCELATIONS_NOTIFY,
  UPDATE_PREF_SERVICE_ERRORS_EXTERNAL,
  UPDATE_PREF_SERVICE_ERRORS_NOTIFY,
  UPDATE_NOT_PREF_UPDATE_AUTO_EXTERNAL,
  UPDATE_NOT_PREF_UPDATE_AUTO_NOTIFY,
  UPDATE_NOT_PREF_UPDATE_ENABLED_EXTERNAL,
  UPDATE_NOT_PREF_UPDATE_ENABLED_NOTIFY,
  UPDATE_NOT_PREF_UPDATE_STAGING_ENABLED_EXTERNAL,
  UPDATE_NOT_PREF_UPDATE_STAGING_ENABLED_NOTIFY,
  UPDATE_NOT_PREF_UPDATE_SERVICE_ENABLED_EXTERNAL,
  UPDATE_NOT_PREF_UPDATE_SERVICE_ENABLED_NOTIFY,
  UPDATE_DOWNLOAD_CODE_COMPLETE,
  UPDATE_DOWNLOAD_CODE_PARTIAL,
  UPDATE_STATE_CODE_COMPLETE_STARTUP,
  UPDATE_STATE_CODE_PARTIAL_STARTUP,
  UPDATE_STATE_CODE_UNKNOWN_STARTUP,
  UPDATE_STATE_CODE_COMPLETE_STAGE,
  UPDATE_STATE_CODE_PARTIAL_STAGE,
  UPDATE_STATE_CODE_UNKNOWN_STAGE,
  UPDATE_STATUS_ERROR_CODE_COMPLETE_STARTUP,
  UPDATE_STATUS_ERROR_CODE_PARTIAL_STARTUP,
  UPDATE_STATUS_ERROR_CODE_UNKNOWN_STARTUP,
  UPDATE_STATUS_ERROR_CODE_COMPLETE_STAGE,
  UPDATE_STATUS_ERROR_CODE_PARTIAL_STAGE,
  UPDATE_STATUS_ERROR_CODE_UNKNOWN_STAGE,
  UPDATE_WIZ_LAST_PAGE_CODE,
  THUNDERBIRD_GLODA_SIZE_MB,
  THUNDERBIRD_CONVERSATIONS_TIME_TO_2ND_GLODA_QUERY_MS,
  THUNDERBIRD_INDEXING_RATE_MSG_PER_S,
  FX_GESTURE_INSTALL_SNAPSHOT_OF_PAGE,
  FX_GESTURE_COMPRESS_SNAPSHOT_OF_PAGE,
  FX_TAB_ANIM_OPEN_PREVIEW_FRAME_INTERVAL_MS,
  FX_TAB_ANIM_OPEN_FRAME_INTERVAL_MS,
  FX_TAB_ANIM_ANY_FRAME_INTERVAL_MS,
  FX_REFRESH_DRIVER_CHROME_FRAME_DELAY_MS,
  FX_REFRESH_DRIVER_CONTENT_FRAME_DELAY_MS,
  FX_REFRESH_DRIVER_SYNC_SCROLL_FRAME_DELAY_MS,
  FX_TAB_SWITCH_UPDATE_MS,
  FX_TAB_SWITCH_TOTAL_MS,
  FX_TAB_SWITCH_TOTAL_E10S_MS,
  FX_TAB_SWITCH_SPINNER_VISIBLE_MS,
  FX_TAB_SWITCH_SPINNER_VISIBLE_LONG_MS,
  FX_TAB_CLICK_MS,
  FX_BOOKMARKS_TOOLBAR_INIT_MS,
  FX_BROWSER_FULLSCREEN_USED,
  FX_NEW_WINDOW_MS,
  FX_PAGE_LOAD_MS,
  FX_TOTAL_TOP_VISITS,
  FX_THUMBNAILS_CAPTURE_TIME_MS,
  FX_THUMBNAILS_STORE_TIME_MS,
  FX_THUMBNAILS_HIT_OR_MISS,
  FX_MIGRATION_ENTRY_POINT,
  FX_MIGRATION_SOURCE_BROWSER,
  FX_MIGRATION_ERRORS,
  FX_MIGRATION_USAGE,
  FX_MIGRATION_IMPORTED_HOMEPAGE,
  FX_MIGRATION_BOOKMARKS_IMPORT_MS,
  FX_MIGRATION_HISTORY_IMPORT_MS,
  FX_MIGRATION_LOGINS_IMPORT_MS,
  FX_STARTUP_MIGRATION_BROWSER_COUNT,
  FX_STARTUP_MIGRATION_EXISTING_DEFAULT_BROWSER,
  FX_STARTUP_MIGRATION_AUTOMATED_IMPORT_PROCESS_SUCCESS,
  FX_STARTUP_MIGRATION_AUTOMATED_IMPORT_UNDO,
  FX_STARTUP_MIGRATION_UNDO_REASON,
  FX_STARTUP_MIGRATION_DATA_RECENCY,
  FX_STARTUP_MIGRATION_USED_RECENT_BROWSER,
  FX_STARTUP_EXTERNAL_CONTENT_HANDLER,
  INPUT_EVENT_RESPONSE_MS,
  EVENTLOOP_UI_ACTIVITY_EXP_MS,
  FX_SESSION_RESTORE_STARTUP_INIT_SESSION_MS,
  FX_SESSION_RESTORE_STARTUP_ONLOAD_INITIAL_WINDOW_MS,
  FX_SESSION_RESTORE_COLLECT_ALL_WINDOWS_DATA_MS,
  FX_SESSION_RESTORE_COLLECT_COOKIES_MS,
  FX_SESSION_RESTORE_COLLECT_DATA_MS,
  FX_SESSION_RESTORE_COLLECT_DATA_LONGEST_OP_MS,
  FX_SESSION_RESTORE_CONTENT_COLLECT_DATA_LONGEST_OP_MS,
  FX_SESSION_RESTORE_SERIALIZE_DATA_MS,
  FX_SESSION_RESTORE_READ_FILE_MS,
  FX_SESSION_RESTORE_WRITE_FILE_MS,
  FX_SESSION_RESTORE_FILE_SIZE_BYTES,
  FX_SESSION_RESTORE_CORRUPT_FILE,
  FX_SESSION_RESTORE_ALL_FILES_CORRUPT,
  FX_SESSION_RESTORE_RESTORE_WINDOW_MS,
  FX_SESSION_RESTORE_SEND_UPDATE_CAUSED_OOM,
  FX_SESSION_RESTORE_DOM_STORAGE_SIZE_ESTIMATE_CHARS,
  FX_SESSION_RESTORE_AUTO_RESTORE_DURATION_UNTIL_EAGER_TABS_RESTORED_MS,
  FX_SESSION_RESTORE_MANUAL_RESTORE_DURATION_UNTIL_EAGER_TABS_RESTORED_MS,
  FX_SESSION_RESTORE_NUMBER_OF_TABS_RESTORED,
  FX_SESSION_RESTORE_NUMBER_OF_WINDOWS_RESTORED,
  FX_SESSION_RESTORE_NUMBER_OF_EAGER_TABS_RESTORED,
  FX_TABLETMODE_PAGE_LOAD,
  FX_TOUCH_USED,
  FX_URLBAR_SELECTED_RESULT_INDEX,
  FX_URLBAR_SELECTED_RESULT_TYPE,
  INNERWINDOWS_WITH_MUTATION_LISTENERS,
  CHARSET_OVERRIDE_SITUATION,
  CHARSET_OVERRIDE_USED,
  DECODER_INSTANTIATED_ISO2022JP,
  DECODER_INSTANTIATED_IBM866,
  DECODER_INSTANTIATED_MACGREEK,
  DECODER_INSTANTIATED_MACICELANDIC,
  DECODER_INSTANTIATED_MACCE,
  DECODER_INSTANTIATED_MACHEBREW,
  DECODER_INSTANTIATED_MACARABIC,
  DECODER_INSTANTIATED_MACFARSI,
  DECODER_INSTANTIATED_MACCROATIAN,
  DECODER_INSTANTIATED_MACCYRILLIC,
  DECODER_INSTANTIATED_MACROMANIAN,
  DECODER_INSTANTIATED_MACTURKISH,
  DECODER_INSTANTIATED_MACDEVANAGARI,
  DECODER_INSTANTIATED_MACGUJARATI,
  DECODER_INSTANTIATED_MACGURMUKHI,
  DECODER_INSTANTIATED_KOI8R,
  DECODER_INSTANTIATED_KOI8U,
  DECODER_INSTANTIATED_ISO_8859_5,
  LONG_REFLOW_INTERRUPTIBLE,
  XMLHTTPREQUEST_ASYNC_OR_SYNC,
  LOCALDOMSTORAGE_SHUTDOWN_DATABASE_MS,
  LOCALDOMSTORAGE_PRELOAD_PENDING_ON_FIRST_ACCESS,
  LOCALDOMSTORAGE_GETALLKEYS_BLOCKING_MS,
  LOCALDOMSTORAGE_GETKEY_BLOCKING_MS,
  LOCALDOMSTORAGE_GETLENGTH_BLOCKING_MS,
  LOCALDOMSTORAGE_GETVALUE_BLOCKING_MS,
  LOCALDOMSTORAGE_SETVALUE_BLOCKING_MS,
  LOCALDOMSTORAGE_REMOVEKEY_BLOCKING_MS,
  LOCALDOMSTORAGE_CLEAR_BLOCKING_MS,
  LOCALDOMSTORAGE_UNLOAD_BLOCKING_MS,
  LOCALDOMSTORAGE_SESSIONONLY_PRELOAD_BLOCKING_MS,
  RANGE_CHECKSUM_ERRORS,
  BUCKET_ORDER_ERRORS,
  TOTAL_COUNT_HIGH_ERRORS,
  TOTAL_COUNT_LOW_ERRORS,
  TELEMETRY_ARCHIVE_DIRECTORIES_COUNT,
  TELEMETRY_ARCHIVE_OLDEST_DIRECTORY_AGE,
  TELEMETRY_ARCHIVE_SCAN_PING_COUNT,
  TELEMETRY_ARCHIVE_SESSION_PING_COUNT,
  TELEMETRY_ARCHIVE_SIZE_MB,
  TELEMETRY_ARCHIVE_EVICTED_OVER_QUOTA,
  TELEMETRY_ARCHIVE_EVICTED_OLD_DIRS,
  TELEMETRY_ARCHIVE_EVICTING_DIRS_MS,
  TELEMETRY_ARCHIVE_CHECKING_OVER_QUOTA_MS,
  TELEMETRY_ARCHIVE_EVICTING_OVER_QUOTA_MS,
  TELEMETRY_PENDING_LOAD_FAILURE_READ,
  TELEMETRY_PENDING_LOAD_FAILURE_PARSE,
  TELEMETRY_PENDING_PINGS_SIZE_MB,
  TELEMETRY_PENDING_PINGS_AGE,
  TELEMETRY_PENDING_PINGS_EVICTED_OVER_QUOTA,
  TELEMETRY_PENDING_EVICTING_OVER_QUOTA_MS,
  TELEMETRY_PENDING_CHECKING_OVER_QUOTA_MS,
  TELEMETRY_PING_SIZE_EXCEEDED_SEND,
  TELEMETRY_PING_SIZE_EXCEEDED_PENDING,
  TELEMETRY_PING_SIZE_EXCEEDED_ARCHIVED,
  TELEMETRY_PING_SUBMISSION_WAITING_CLIENTID,
  TELEMETRY_DISCARDED_PENDING_PINGS_SIZE_MB,
  TELEMETRY_DISCARDED_ARCHIVED_PINGS_SIZE_MB,
  TELEMETRY_DISCARDED_SEND_PINGS_SIZE_MB,
  TELEMETRY_DISCARDED_CONTENT_PINGS_COUNT,
  TELEMETRY_COMPRESS,
  TELEMETRY_PING,
  TELEMETRY_SEND,
  TELEMETRY_STRINGIFY,
  TELEMETRY_SUCCESS,
  TELEMETRY_INVALID_PING_TYPE_SUBMITTED,
  TELEMETRY_PING_EVICTED_FOR_SERVER_ERRORS,
  TELEMETRY_SESSIONDATA_FAILED_LOAD,
  TELEMETRY_SESSIONDATA_FAILED_PARSE,
  TELEMETRY_SESSIONDATA_FAILED_VALIDATION,
  TELEMETRY_SESSIONDATA_FAILED_SAVE,
  TELEMETRY_ASSEMBLE_PAYLOAD_EXCEPTION,
  TELEMETRY_SCHEDULER_TICK_EXCEPTION,
  TELEMETRY_SCHEDULER_WAKEUP,
  TELEMETRY_SCHEDULER_SEND_DAILY,
  TELEMETRY_TEST_FLAG,
  TELEMETRY_TEST_COUNT,
  TELEMETRY_TEST_COUNT_INIT_NO_RECORD,
  TELEMETRY_TEST_CATEGORICAL,
  TELEMETRY_TEST_CATEGORICAL_OPTOUT,
  TELEMETRY_TEST_KEYED_COUNT_INIT_NO_RECORD,
  TELEMETRY_TEST_KEYED_FLAG,
  TELEMETRY_TEST_KEYED_COUNT,
  TELEMETRY_TEST_RELEASE_OPTOUT,
  TELEMETRY_TEST_RELEASE_OPTIN,
  TELEMETRY_TEST_KEYED_RELEASE_OPTIN,
  TELEMETRY_TEST_KEYED_RELEASE_OPTOUT,
  STARTUP_CRASH_DETECTED,
  SAFE_MODE_USAGE,
  XCTO_NOSNIFF_BLOCK_IMAGE,
  NEWTAB_PAGE_ENABLED,
  NEWTAB_PAGE_ENHANCED,
  NEWTAB_PAGE_LIFE_SPAN,
  NEWTAB_PAGE_LIFE_SPAN_SUGGESTED,
  NEWTAB_PAGE_PINNED_SITES_COUNT,
  NEWTAB_PAGE_BLOCKED_SITES_COUNT,
  NEWTAB_PAGE_SHOWN,
  NEWTAB_PAGE_SITE_CLICKED,
#if defined(ANDROID)
  BROWSERPROVIDER_XUL_IMPORT_BOOKMARKS,
#endif
#if defined(ANDROID)
  FENNEC_GLOBALHISTORY_ADD_MS,
#endif
#if defined(ANDROID)
  FENNEC_GLOBALHISTORY_UPDATE_MS,
#endif
#if defined(ANDROID)
  FENNEC_GLOBALHISTORY_VISITED_BUILD_MS,
#endif
#if defined(ANDROID)
  FENNEC_RESTORING_ACTIVITY,
#endif
#if defined(ANDROID)
  FENNEC_SEARCH_LOADER_TIME_MS,
#endif
#if defined(ANDROID)
  FENNEC_STARTUP_TIME_GECKOREADY,
#endif
#if defined(ANDROID)
  FENNEC_STARTUP_TIME_JAVAUI,
#endif
#if defined(ANDROID)
  FENNEC_TOPSITES_LOADER_TIME_MS,
#endif
#if defined(ANDROID)
  FENNEC_HOMEPANELS_CUSTOM,
#endif
#if defined(ANDROID)
  FENNEC_WAS_KILLED,
#endif
  FIPS_ENABLED,
  SECURITY_UI,
  JS_TELEMETRY_ADDON_EXCEPTIONS,
  IPC_TRANSACTION_CANCEL,
  IPC_SAME_PROCESS_MESSAGE_COPY_OOM_KB,
  SLOW_ADDON_WARNING_STATES,
  SLOW_ADDON_WARNING_RESPONSE_TIME,
  SEARCH_COUNTS,
  SEARCH_RESET_RESULT,
  SEARCH_SERVICE_INIT_MS,
  SEARCH_SERVICE_INIT_SYNC,
  SEARCH_SERVICE_ENGINE_COUNT,
  SEARCH_SERVICE_HAS_UPDATES,
  SEARCH_SERVICE_HAS_ICON_UPDATES,
  SEARCH_SERVICE_COUNTRY_FETCH_TIME_MS,
  SEARCH_SERVICE_COUNTRY_FETCH_RESULT,
  SEARCH_SERVICE_COUNTRY_TIMEOUT,
  SEARCH_SERVICE_COUNTRY_FETCH_CAUSED_SYNC_INIT,
  SEARCH_SERVICE_US_COUNTRY_MISMATCHED_TIMEZONE,
  SEARCH_SERVICE_US_TIMEZONE_MISMATCHED_COUNTRY,
  SEARCH_SERVICE_US_COUNTRY_MISMATCHED_PLATFORM_OSX,
  SEARCH_SERVICE_NONUS_COUNTRY_MISMATCHED_PLATFORM_OSX,
  SEARCH_SERVICE_US_COUNTRY_MISMATCHED_PLATFORM_WIN,
  SEARCH_SERVICE_NONUS_COUNTRY_MISMATCHED_PLATFORM_WIN,
  SOCIAL_ENABLED_ON_SESSION,
  ENABLE_PRIVILEGE_EVER_CALLED,
  SUBJECT_PRINCIPAL_ACCESSED_WITHOUT_SCRIPT_ON_STACK,
#if defined(XP_WIN)
  TOUCH_ENABLED_DEVICE,
#endif
  COMPONENTS_SHIM_ACCESSED_BY_CONTENT,
  CHECK_ADDONS_MODIFIED_MS,
  TELEMETRY_MEMORY_REPORTER_MS,
  SSL_SUCCESFUL_CERT_VALIDATION_TIME_MOZILLAPKIX,
  SSL_INITIAL_FAILED_CERT_VALIDATION_TIME_MOZILLAPKIX,
  CRASH_STORE_COMPRESSED_BYTES,
  PDF_VIEWER_USED,
  PDF_VIEWER_FALLBACK_SHOWN,
  PDF_VIEWER_PRINT,
  PDF_VIEWER_DOCUMENT_VERSION,
  PDF_VIEWER_DOCUMENT_GENERATOR,
  PDF_VIEWER_DOCUMENT_SIZE_KB,
  PDF_VIEWER_FONT_TYPES,
  PDF_VIEWER_EMBED,
  PDF_VIEWER_FORM,
  PDF_VIEWER_STREAM_TYPES,
  PDF_VIEWER_TIME_TO_VIEW_MS,
  PLUGINS_NOTIFICATION_SHOWN,
  PLUGINS_NOTIFICATION_PLUGIN_COUNT,
  PLUGINS_NOTIFICATION_USER_ACTION,
  PLUGINS_INFOBAR_SHOWN,
  PLUGINS_INFOBAR_BLOCK,
  PLUGINS_INFOBAR_ALLOW,
  POPUP_NOTIFICATION_STATS,
  POPUP_NOTIFICATION_MAIN_ACTION_MS,
  POPUP_NOTIFICATION_DISMISSAL_MS,
  PRINT_PREVIEW_OPENED_COUNT,
  PRINT_PREVIEW_SIMPLIFY_PAGE_OPENED_COUNT,
  PRINT_PREVIEW_SIMPLIFY_PAGE_UNAVAILABLE_COUNT,
  PRINT_COUNT,
  DEVTOOLS_DEBUGGER_DISPLAY_SOURCE_LOCAL_MS,
  DEVTOOLS_DEBUGGER_DISPLAY_SOURCE_REMOTE_MS,
#if defined(MOZ_RUST_MP4PARSE)
  MEDIA_RUST_MP4PARSE_SUCCESS,
#endif
#if defined(MOZ_RUST_MP4PARSE)
  MEDIA_RUST_MP4PARSE_ERROR_CODE,
#endif
#if defined(MOZ_RUST_MP4PARSE)
  MEDIA_RUST_MP4PARSE_TRACK_MATCH_AUDIO,
#endif
#if defined(MOZ_RUST_MP4PARSE)
  MEDIA_RUST_MP4PARSE_TRACK_MATCH_VIDEO,
#endif
  MEDIA_WMF_DECODE_ERROR,
  MEDIA_OGG_LOADED_IS_CHAINED,
  MEDIA_HLS_CANPLAY_REQUESTED,
  MEDIA_HLS_DECODER_SUCCESS,
  VIDEO_MFT_OUTPUT_NULL_SAMPLES,
  AUDIO_MFT_OUTPUT_NULL_SAMPLES,
  VIDEO_CAN_CREATE_AAC_DECODER,
  VIDEO_CAN_CREATE_H264_DECODER,
  VIDEO_CANPLAYTYPE_H264_CONSTRAINT_SET_FLAG,
  VIDEO_CANPLAYTYPE_H264_LEVEL,
  VIDEO_CANPLAYTYPE_H264_PROFILE,
  DECODER_DOCTOR_INFOBAR_STATS,
  VIDEO_DECODED_H264_SPS_CONSTRAINT_SET_FLAG,
  VIDEO_DECODED_H264_SPS_LEVEL,
  VIDEO_DECODED_H264_SPS_PROFILE,
  VIDEO_H264_SPS_MAX_NUM_REF_FRAMES,
  WEBRTC_ICE_FINAL_CONNECTION_STATE,
  WEBRTC_ICE_ON_TIME_TRICKLE_ARRIVAL_TIME,
  WEBRTC_ICE_LATE_TRICKLE_ARRIVAL_TIME,
  WEBRTC_ICE_SUCCESS_TIME,
  WEBRTC_ICE_FAILURE_TIME,
  WEBRTC_ICE_SUCCESS_RATE,
  WEBRTC_STUN_RATE_LIMIT_EXCEEDED_BY_TYPE_GIVEN_SUCCESS,
  WEBRTC_STUN_RATE_LIMIT_EXCEEDED_BY_TYPE_GIVEN_FAILURE,
  WEBRTC_AVSYNC_WHEN_AUDIO_LAGS_VIDEO_MS,
  WEBRTC_AVSYNC_WHEN_VIDEO_LAGS_AUDIO_MS,
  WEBRTC_VIDEO_QUALITY_INBOUND_BANDWIDTH_KBITS,
  WEBRTC_AUDIO_QUALITY_INBOUND_BANDWIDTH_KBITS,
  WEBRTC_VIDEO_QUALITY_OUTBOUND_BANDWIDTH_KBITS,
  WEBRTC_AUDIO_QUALITY_OUTBOUND_BANDWIDTH_KBITS,
  WEBRTC_VIDEO_QUALITY_INBOUND_PACKETLOSS_RATE,
  WEBRTC_AUDIO_QUALITY_INBOUND_PACKETLOSS_RATE,
  WEBRTC_VIDEO_QUALITY_OUTBOUND_PACKETLOSS_RATE,
  WEBRTC_AUDIO_QUALITY_OUTBOUND_PACKETLOSS_RATE,
  WEBRTC_VIDEO_QUALITY_INBOUND_JITTER,
  WEBRTC_AUDIO_QUALITY_INBOUND_JITTER,
  WEBRTC_VIDEO_QUALITY_OUTBOUND_JITTER,
  WEBRTC_AUDIO_QUALITY_OUTBOUND_JITTER,
  WEBRTC_VIDEO_ERROR_RECOVERY_MS,
  WEBRTC_VIDEO_RECOVERY_BEFORE_ERROR_PER_MIN,
  WEBRTC_VIDEO_RECOVERY_AFTER_ERROR_PER_MIN,
  WEBRTC_VIDEO_DECODE_ERROR_TIME_PERMILLE,
  WEBRTC_VIDEO_QUALITY_OUTBOUND_RTT,
  WEBRTC_AUDIO_QUALITY_OUTBOUND_RTT,
  WEBRTC_VIDEO_ENCODER_BITRATE_AVG_PER_CALL_KBPS,
  WEBRTC_VIDEO_ENCODER_BITRATE_STD_DEV_PER_CALL_KBPS,
  WEBRTC_VIDEO_ENCODER_FRAMERATE_AVG_PER_CALL,
  WEBRTC_VIDEO_ENCODER_FRAMERATE_10X_STD_DEV_PER_CALL,
  WEBRTC_VIDEO_ENCODER_DROPPED_FRAMES_PER_CALL_FPM,
  WEBRTC_VIDEO_DECODER_BITRATE_AVG_PER_CALL_KBPS,
  WEBRTC_VIDEO_DECODER_BITRATE_STD_DEV_PER_CALL_KBPS,
  WEBRTC_VIDEO_DECODER_FRAMERATE_AVG_PER_CALL,
  WEBRTC_VIDEO_DECODER_FRAMERATE_10X_STD_DEV_PER_CALL,
  WEBRTC_VIDEO_DECODER_DISCARDED_PACKETS_PER_CALL_PPM,
  WEBRTC_CALL_DURATION,
  WEBRTC_CALL_COUNT,
  WEBRTC_CALL_COUNT_2,
  WEBRTC_ICE_ADD_CANDIDATE_ERRORS_GIVEN_SUCCESS,
  WEBRTC_ICE_ADD_CANDIDATE_ERRORS_GIVEN_FAILURE,
  WEBRTC_GET_USER_MEDIA_SECURE_ORIGIN,
  WEBRTC_GET_USER_MEDIA_TYPE,
  WEBRTC_LOAD_STATE_RELAXED,
  WEBRTC_LOAD_STATE_RELAXED_SHORT,
  WEBRTC_LOAD_STATE_NORMAL,
  WEBRTC_LOAD_STATE_NORMAL_SHORT,
  WEBRTC_LOAD_STATE_STRESSED,
  WEBRTC_LOAD_STATE_STRESSED_SHORT,
  WEBRTC_RENEGOTIATIONS,
  WEBRTC_MAX_VIDEO_SEND_TRACK,
  WEBRTC_MAX_VIDEO_RECEIVE_TRACK,
  WEBRTC_MAX_AUDIO_SEND_TRACK,
  WEBRTC_MAX_AUDIO_RECEIVE_TRACK,
  WEBRTC_DATACHANNEL_NEGOTIATED,
  WEBRTC_CALL_TYPE,
  DEVTOOLS_TOOLBOX_OPENED_COUNT,
  DEVTOOLS_OPTIONS_OPENED_COUNT,
  DEVTOOLS_WEBCONSOLE_OPENED_COUNT,
  DEVTOOLS_BROWSERCONSOLE_OPENED_COUNT,
  DEVTOOLS_INSPECTOR_OPENED_COUNT,
  DEVTOOLS_RULEVIEW_OPENED_COUNT,
  DEVTOOLS_COMPUTEDVIEW_OPENED_COUNT,
  DEVTOOLS_FONTINSPECTOR_OPENED_COUNT,
  DEVTOOLS_ANIMATIONINSPECTOR_OPENED_COUNT,
  DEVTOOLS_JSDEBUGGER_OPENED_COUNT,
  DEVTOOLS_JSBROWSERDEBUGGER_OPENED_COUNT,
  DEVTOOLS_STYLEEDITOR_OPENED_COUNT,
  DEVTOOLS_SHADEREDITOR_OPENED_COUNT,
  DEVTOOLS_WEBAUDIOEDITOR_OPENED_COUNT,
  DEVTOOLS_CANVASDEBUGGER_OPENED_COUNT,
  DEVTOOLS_JSPROFILER_OPENED_COUNT,
  DEVTOOLS_MEMORY_OPENED_COUNT,
  DEVTOOLS_NETMONITOR_OPENED_COUNT,
  DEVTOOLS_STORAGE_OPENED_COUNT,
  DEVTOOLS_PAINTFLASHING_OPENED_COUNT,
  DEVTOOLS_TILT_OPENED_COUNT,
  DEVTOOLS_SCRATCHPAD_OPENED_COUNT,
  DEVTOOLS_SCRATCHPAD_WINDOW_OPENED_COUNT,
  DEVTOOLS_RESPONSIVE_OPENED_COUNT,
  DEVTOOLS_EYEDROPPER_OPENED_COUNT,
  DEVTOOLS_MENU_EYEDROPPER_OPENED_COUNT,
  DEVTOOLS_PICKER_EYEDROPPER_OPENED_COUNT,
  DEVTOOLS_DEVELOPERTOOLBAR_OPENED_COUNT,
  DEVTOOLS_ABOUTDEBUGGING_OPENED_COUNT,
  DEVTOOLS_WEBIDE_OPENED_COUNT,
  DEVTOOLS_WEBIDE_PROJECT_EDITOR_OPENED_COUNT,
  DEVTOOLS_WEBIDE_PROJECT_EDITOR_SAVE_COUNT,
  DEVTOOLS_WEBIDE_NEW_PROJECT_COUNT,
  DEVTOOLS_WEBIDE_IMPORT_PROJECT_COUNT,
  DEVTOOLS_CUSTOM_OPENED_COUNT,
  DEVTOOLS_RELOAD_ADDON_INSTALLED_COUNT,
  DEVTOOLS_RELOAD_ADDON_RELOAD_COUNT,
  DEVTOOLS_TOOLBOX_OPENED_PER_USER_FLAG,
  DEVTOOLS_OPTIONS_OPENED_PER_USER_FLAG,
  DEVTOOLS_WEBCONSOLE_OPENED_PER_USER_FLAG,
  DEVTOOLS_BROWSERCONSOLE_OPENED_PER_USER_FLAG,
  DEVTOOLS_INSPECTOR_OPENED_PER_USER_FLAG,
  DEVTOOLS_RULEVIEW_OPENED_PER_USER_FLAG,
  DEVTOOLS_COMPUTEDVIEW_OPENED_PER_USER_FLAG,
  DEVTOOLS_FONTINSPECTOR_OPENED_PER_USER_FLAG,
  DEVTOOLS_ANIMATIONINSPECTOR_OPENED_PER_USER_FLAG,
  DEVTOOLS_JSDEBUGGER_OPENED_PER_USER_FLAG,
  DEVTOOLS_JSBROWSERDEBUGGER_OPENED_PER_USER_FLAG,
  DEVTOOLS_STYLEEDITOR_OPENED_PER_USER_FLAG,
  DEVTOOLS_SHADEREDITOR_OPENED_PER_USER_FLAG,
  DEVTOOLS_WEBAUDIOEDITOR_OPENED_PER_USER_FLAG,
  DEVTOOLS_CANVASDEBUGGER_OPENED_PER_USER_FLAG,
  DEVTOOLS_JSPROFILER_OPENED_PER_USER_FLAG,
  DEVTOOLS_MEMORY_OPENED_PER_USER_FLAG,
  DEVTOOLS_NETMONITOR_OPENED_PER_USER_FLAG,
  DEVTOOLS_STORAGE_OPENED_PER_USER_FLAG,
  DEVTOOLS_PAINTFLASHING_OPENED_PER_USER_FLAG,
  DEVTOOLS_TILT_OPENED_PER_USER_FLAG,
  DEVTOOLS_SCRATCHPAD_OPENED_PER_USER_FLAG,
  DEVTOOLS_SCRATCHPAD_WINDOW_OPENED_PER_USER_FLAG,
  DEVTOOLS_RESPONSIVE_OPENED_PER_USER_FLAG,
  DEVTOOLS_EYEDROPPER_OPENED_PER_USER_FLAG,
  DEVTOOLS_MENU_EYEDROPPER_OPENED_PER_USER_FLAG,
  DEVTOOLS_PICKER_EYEDROPPER_OPENED_PER_USER_FLAG,
  DEVTOOLS_DEVELOPERTOOLBAR_OPENED_PER_USER_FLAG,
  DEVTOOLS_ABOUTDEBUGGING_OPENED_PER_USER_FLAG,
  DEVTOOLS_WEBIDE_OPENED_PER_USER_FLAG,
  DEVTOOLS_WEBIDE_PROJECT_EDITOR_OPENED_PER_USER_FLAG,
  DEVTOOLS_WEBIDE_PROJECT_EDITOR_SAVE_PER_USER_FLAG,
  DEVTOOLS_WEBIDE_NEW_PROJECT_PER_USER_FLAG,
  DEVTOOLS_WEBIDE_IMPORT_PROJECT_PER_USER_FLAG,
  DEVTOOLS_CUSTOM_OPENED_PER_USER_FLAG,
  DEVTOOLS_RELOAD_ADDON_INSTALLED_PER_USER_FLAG,
  DEVTOOLS_RELOAD_ADDON_RELOAD_PER_USER_FLAG,
  DEVTOOLS_TOOLBOX_TIME_ACTIVE_SECONDS,
  DEVTOOLS_OPTIONS_TIME_ACTIVE_SECONDS,
  DEVTOOLS_WEBCONSOLE_TIME_ACTIVE_SECONDS,
  DEVTOOLS_BROWSERCONSOLE_TIME_ACTIVE_SECONDS,
  DEVTOOLS_INSPECTOR_TIME_ACTIVE_SECONDS,
  DEVTOOLS_RULEVIEW_TIME_ACTIVE_SECONDS,
  DEVTOOLS_COMPUTEDVIEW_TIME_ACTIVE_SECONDS,
  DEVTOOLS_FONTINSPECTOR_TIME_ACTIVE_SECONDS,
  DEVTOOLS_ANIMATIONINSPECTOR_TIME_ACTIVE_SECONDS,
  DEVTOOLS_JSDEBUGGER_TIME_ACTIVE_SECONDS,
  DEVTOOLS_JSBROWSERDEBUGGER_TIME_ACTIVE_SECONDS,
  DEVTOOLS_STYLEEDITOR_TIME_ACTIVE_SECONDS,
  DEVTOOLS_SHADEREDITOR_TIME_ACTIVE_SECONDS,
  DEVTOOLS_WEBAUDIOEDITOR_TIME_ACTIVE_SECONDS,
  DEVTOOLS_CANVASDEBUGGER_TIME_ACTIVE_SECONDS,
  DEVTOOLS_JSPROFILER_TIME_ACTIVE_SECONDS,
  DEVTOOLS_MEMORY_TIME_ACTIVE_SECONDS,
  DEVTOOLS_NETMONITOR_TIME_ACTIVE_SECONDS,
  DEVTOOLS_STORAGE_TIME_ACTIVE_SECONDS,
  DEVTOOLS_PAINTFLASHING_TIME_ACTIVE_SECONDS,
  DEVTOOLS_TILT_TIME_ACTIVE_SECONDS,
  DEVTOOLS_SCRATCHPAD_TIME_ACTIVE_SECONDS,
  DEVTOOLS_SCRATCHPAD_WINDOW_TIME_ACTIVE_SECONDS,
  DEVTOOLS_RESPONSIVE_TIME_ACTIVE_SECONDS,
  DEVTOOLS_DEVELOPERTOOLBAR_TIME_ACTIVE_SECONDS,
  DEVTOOLS_ABOUTDEBUGGING_TIME_ACTIVE_SECONDS,
  DEVTOOLS_WEBIDE_TIME_ACTIVE_SECONDS,
  DEVTOOLS_WEBIDE_PROJECT_EDITOR_TIME_ACTIVE_SECONDS,
  DEVTOOLS_CUSTOM_TIME_ACTIVE_SECONDS,
  DEVTOOLS_WEBIDE_CONNECTION_RESULT,
  DEVTOOLS_WEBIDE_USB_CONNECTION_RESULT,
  DEVTOOLS_WEBIDE_WIFI_CONNECTION_RESULT,
  DEVTOOLS_WEBIDE_SIMULATOR_CONNECTION_RESULT,
  DEVTOOLS_WEBIDE_REMOTE_CONNECTION_RESULT,
  DEVTOOLS_WEBIDE_LOCAL_CONNECTION_RESULT,
  DEVTOOLS_WEBIDE_OTHER_CONNECTION_RESULT,
  DEVTOOLS_WEBIDE_CONNECTION_TIME_SECONDS,
  DEVTOOLS_WEBIDE_CONNECTION_PLAY_USED,
  DEVTOOLS_WEBIDE_CONNECTION_DEBUG_USED,
  DEVTOOLS_WEBIDE_CONNECTED_RUNTIME_TYPE,
  DEVTOOLS_WEBIDE_CONNECTED_RUNTIME_ID,
  DEVTOOLS_WEBIDE_CONNECTED_RUNTIME_PROCESSOR,
  DEVTOOLS_WEBIDE_CONNECTED_RUNTIME_OS,
  DEVTOOLS_WEBIDE_CONNECTED_RUNTIME_PLATFORM_VERSION,
  DEVTOOLS_WEBIDE_CONNECTED_RUNTIME_APP_TYPE,
  DEVTOOLS_WEBIDE_CONNECTED_RUNTIME_VERSION,
  DEVTOOLS_OS_ENUMERATED_PER_USER,
  DEVTOOLS_OS_IS_64_BITS_PER_USER,
  DEVTOOLS_SCREEN_RESOLUTION_ENUMERATED_PER_USER,
  DEVTOOLS_TABS_OPEN_PEAK_LINEAR,
  DEVTOOLS_TABS_OPEN_AVERAGE_LINEAR,
  DEVTOOLS_TABS_PINNED_PEAK_LINEAR,
  DEVTOOLS_TABS_PINNED_AVERAGE_LINEAR,
  DEVTOOLS_SAVE_HEAP_SNAPSHOT_MS,
  DEVTOOLS_READ_HEAP_SNAPSHOT_MS,
  DEVTOOLS_HEAP_SNAPSHOT_NODE_COUNT,
  DEVTOOLS_HEAP_SNAPSHOT_EDGE_COUNT,
  DEVTOOLS_PERFTOOLS_RECORDING_COUNT,
  DEVTOOLS_PERFTOOLS_CONSOLE_RECORDING_COUNT,
  DEVTOOLS_PERFTOOLS_RECORDING_IMPORT_FLAG,
  DEVTOOLS_PERFTOOLS_RECORDING_EXPORT_FLAG,
  DEVTOOLS_PERFTOOLS_RECORDING_FEATURES_USED,
  DEVTOOLS_PERFTOOLS_RECORDING_DURATION_MS,
  DEVTOOLS_PERFTOOLS_SELECTED_VIEW_MS,
  DEVTOOLS_JAVASCRIPT_ERROR_DISPLAYED,
  VIEW_SOURCE_IN_BROWSER_OPENED_BOOLEAN,
  VIEW_SOURCE_IN_WINDOW_OPENED_BOOLEAN,
  VIEW_SOURCE_EXTERNAL_RESULT_BOOLEAN,
  BROWSER_IS_USER_DEFAULT,
  BROWSER_IS_USER_DEFAULT_ERROR,
  BROWSER_SET_DEFAULT_DIALOG_PROMPT_RAWCOUNT,
  BROWSER_SET_DEFAULT_ALWAYS_CHECK,
  BROWSER_SET_DEFAULT_RESULT,
  BROWSER_SET_DEFAULT_ERROR,
  BROWSER_SET_DEFAULT_TIME_TO_COMPLETION_SECONDS,
  BROWSER_IS_ASSIST_DEFAULT,
  MIXED_CONTENT_PAGE_LOAD,
  MIXED_CONTENT_UNBLOCK_COUNTER,
  MIXED_CONTENT_HSTS,
  MIXED_CONTENT_OBJECT_SUBREQUEST,
  COOKIE_SCHEME_SECURITY,
  NTLM_MODULE_USED_2,
  FX_THUMBNAILS_BG_QUEUE_SIZE_ON_CAPTURE,
  FX_THUMBNAILS_BG_CAPTURE_QUEUE_TIME_MS,
  FX_THUMBNAILS_BG_CAPTURE_SERVICE_TIME_MS,
  FX_THUMBNAILS_BG_CAPTURE_DONE_REASON_2,
  FX_THUMBNAILS_BG_CAPTURE_PAGE_LOAD_TIME_MS,
  FX_THUMBNAILS_BG_CAPTURE_CANVAS_DRAW_TIME_MS,
  NETWORK_CACHE_V2_MISS_TIME_MS,
  NETWORK_CACHE_V2_HIT_TIME_MS,
  NETWORK_CACHE_V1_TRUNCATE_TIME_MS,
  NETWORK_CACHE_V1_MISS_TIME_MS,
  NETWORK_CACHE_V1_HIT_TIME_MS,
  NETWORK_CACHE_V2_OUTPUT_STREAM_STATUS,
  NETWORK_CACHE_V2_INPUT_STREAM_STATUS,
  NETWORK_CACHE_FS_TYPE,
  NETWORK_CACHE_SIZE_FULL_FAT,
  NETWORK_CACHE_HIT_MISS_STAT_PER_CACHE_SIZE,
  NETWORK_CACHE_HIT_RATE_PER_CACHE_SIZE,
  NETWORK_CACHE_METADATA_FIRST_READ_TIME_MS,
  NETWORK_CACHE_METADATA_SECOND_READ_TIME_MS,
  NETWORK_CACHE_METADATA_FIRST_READ_SIZE,
  NETWORK_CACHE_METADATA_SIZE,
  NETWORK_CACHE_HASH_STATS,
  DATABASE_LOCKED_EXCEPTION,
  DATABASE_SUCCESSFUL_UNLOCK,
  SSL_TLS13_INTOLERANCE_REASON_PRE,
  SSL_TLS13_INTOLERANCE_REASON_POST,
  SSL_TLS12_INTOLERANCE_REASON_PRE,
  SSL_TLS12_INTOLERANCE_REASON_POST,
  SSL_TLS11_INTOLERANCE_REASON_PRE,
  SSL_TLS11_INTOLERANCE_REASON_POST,
  SSL_TLS10_INTOLERANCE_REASON_PRE,
  SSL_TLS10_INTOLERANCE_REASON_POST,
  SSL_VERSION_FALLBACK_INAPPROPRIATE,
  SSL_WEAK_CIPHERS_FALLBACK,
  SSL_CIPHER_SUITE_FULL,
  SSL_CIPHER_SUITE_RESUMED,
  SSL_KEA_RSA_KEY_SIZE_FULL,
  SSL_KEA_DHE_KEY_SIZE_FULL,
  SSL_KEA_ECDHE_CURVE_FULL,
  SSL_AUTH_ALGORITHM_FULL,
  SSL_AUTH_RSA_KEY_SIZE_FULL,
  SSL_AUTH_ECDSA_CURVE_FULL,
  SSL_SYMMETRIC_CIPHER_FULL,
  SSL_SYMMETRIC_CIPHER_RESUMED,
  SSL_REASONS_FOR_NOT_FALSE_STARTING,
  SSL_HANDSHAKE_TYPE,
  SSL_OCSP_STAPLING,
  SSL_OCSP_MAY_FETCH,
  SSL_CERT_ERROR_OVERRIDES,
  SSL_CERT_VERIFICATION_ERRORS,
  SSL_PERMANENT_CERT_ERROR_OVERRIDES,
  SSL_SERVER_AUTH_EKU,
  TELEMETRY_TEST_EXPIRED,
  TLS_ERROR_REPORT_UI,
  CERT_OCSP_ENABLED,
  CERT_OCSP_REQUIRED,
  OSFILE_WORKER_LAUNCH_MS,
  OSFILE_WORKER_READY_MS,
  OSFILE_WRITEATOMIC_JANK_MS,
  CERT_EV_STATUS,
  CERT_VALIDATION_SUCCESS_BY_CA,
  CERT_PINNING_FAILURES_BY_CA,
  CERT_PINNING_RESULTS,
  CERT_PINNING_TEST_RESULTS,
  CERT_PINNING_MOZ_RESULTS,
  CERT_PINNING_MOZ_TEST_RESULTS,
  CERT_PINNING_MOZ_RESULTS_BY_HOST,
  CERT_PINNING_MOZ_TEST_RESULTS_BY_HOST,
  CERT_CHAIN_KEY_SIZE_STATUS,
  CERT_CHAIN_SHA1_POLICY_STATUS,
  WEAVE_CONFIGURED,
  WEAVE_CONFIGURED_MASTER_PASSWORD,
  WEAVE_START_COUNT,
  WEAVE_COMPLETE_SUCCESS_COUNT,
  WEBCRYPTO_EXTRACTABLE_IMPORT,
  WEBCRYPTO_EXTRACTABLE_GENERATE,
  WEBCRYPTO_EXTRACTABLE_ENC,
  WEBCRYPTO_EXTRACTABLE_SIG,
  WEBCRYPTO_RESOLVED,
  WEBCRYPTO_METHOD,
  WEBCRYPTO_ALG,
  MASTER_PASSWORD_ENABLED,
#if defined(XP_MACOSX)
  DISPLAY_SCALING_OSX,
#endif
#if defined(XP_WIN)
  DISPLAY_SCALING_MSWIN,
#endif
#if defined(XP_LINUX)
  DISPLAY_SCALING_LINUX,
#endif
  SOCIAL_SIDEBAR_STATE,
  SOCIAL_TOOLBAR_BUTTONS,
  SOCIAL_PANEL_CLICKS,
  SOCIAL_SIDEBAR_OPEN_DURATION,
  SHUTDOWN_PHASE_DURATION_TICKS_QUIT_APPLICATION,
  SHUTDOWN_PHASE_DURATION_TICKS_PROFILE_CHANGE_TEARDOWN,
  SHUTDOWN_PHASE_DURATION_TICKS_XPCOM_WILL_SHUTDOWN,
  SHUTDOWN_PHASE_DURATION_TICKS_PROFILE_BEFORE_CHANGE,
  BR_9_2_1_SUBJECT_ALT_NAMES,
  BR_9_2_2_SUBJECT_COMMON_NAME,
  TAP_TO_LOAD_ENABLED,
  ZOOMED_VIEW_ENABLED,
  TRACKING_PROTECTION_ENABLED,
  TRACKING_PROTECTION_PBM_DISABLED,
  FENNEC_TRACKING_PROTECTION_STATE,
  TRACKING_PROTECTION_SHIELD,
  TRACKING_PROTECTION_EVENTS,
  SERVICE_WORKER_REGISTRATION_LOADING,
  SERVICE_WORKER_REQUEST_PASSTHROUGH,
  LOOP_CLIENT_CALL_URL_REQUESTS_SUCCESS,
  LOOP_CLIENT_CALL_URL_SHARED,
  LOOP_ROOM_CREATE,
  LOOP_COPY_PANEL_ACTIONS,
  LOOP_ACTIVITY_COUNTER,
  E10S_STATUS,
  E10S_WINDOW,
  E10S_BLOCKED_FROM_RUNNING,
  BLOCKED_ON_PLUGIN_MODULE_INIT_MS,
  BLOCKED_ON_PLUGIN_INSTANCE_INIT_MS,
  BLOCKED_ON_PLUGIN_STREAM_INIT_MS,
  BLOCKED_ON_PLUGINASYNCSURROGATE_WAITFORINIT_MS,
  BLOCKED_ON_PLUGIN_INSTANCE_DESTROY_MS,
  ONBEFOREUNLOAD_PROMPT_ACTION,
  ONBEFOREUNLOAD_PROMPT_COUNT,
  SUBPROCESS_ABNORMAL_ABORT,
  SUBPROCESS_CRASHES_WITH_DUMP,
  SUBPROCESS_LAUNCH_FAILURE,
  PROCESS_CRASH_SUBMIT_ATTEMPT,
  PROCESS_CRASH_SUBMIT_SUCCESS,
  STUMBLER_TIME_BETWEEN_UPLOADS_SEC,
  STUMBLER_VOLUME_BYTES_UPLOADED_PER_SEC,
  STUMBLER_TIME_BETWEEN_START_SEC,
  STUMBLER_UPLOAD_BYTES,
  STUMBLER_UPLOAD_OBSERVATION_COUNT,
  STUMBLER_UPLOAD_CELL_COUNT,
  STUMBLER_UPLOAD_WIFI_AP_COUNT,
  STUMBLER_OBSERVATIONS_PER_DAY,
  STUMBLER_TIME_BETWEEN_RECEIVED_LOCATIONS_SEC,
  DATA_STORAGE_ENTRIES,
  VIDEO_EME_PLAY_SUCCESS,
  VIDEO_PLAY_TIME_MS,
  VIDEO_HIDDEN_PLAY_TIME_MS,
  VIDEO_HIDDEN_PLAY_TIME_PERCENTAGE,
  VIDEO_INTER_KEYFRAME_AVERAGE_MS,
  VIDEO_INTER_KEYFRAME_MAX_MS,
  VIDEO_UNLOAD_STATE,
  VIDEO_VP9_BENCHMARK_FPS,
  VIDEO_CDM_CREATED,
  VIDEO_CDM_GENERATE_REQUEST_CALLED,
  MEDIA_CODEC_USED,
  FX_SANITIZE_TOTAL,
  FX_SANITIZE_CACHE,
  FX_SANITIZE_COOKIES_2,
  FX_SANITIZE_LOADED_FLASH,
  FX_SANITIZE_UNLOADED_FLASH,
  FX_SANITIZE_HISTORY,
  FX_SANITIZE_FORMDATA,
  FX_SANITIZE_DOWNLOADS,
  FX_SANITIZE_SESSIONS,
  FX_SANITIZE_SITESETTINGS,
  FX_SANITIZE_OPENWINDOWS,
  PWMGR_BLOCKLIST_NUM_SITES,
  PWMGR_FORM_AUTOFILL_RESULT,
  PWMGR_LOGIN_LAST_USED_DAYS,
  PWMGR_LOGIN_PAGE_SAFETY,
  PWMGR_MANAGE_COPIED_PASSWORD,
  PWMGR_MANAGE_COPIED_USERNAME,
  PWMGR_MANAGE_DELETED,
  PWMGR_MANAGE_DELETED_ALL,
  PWMGR_MANAGE_OPENED,
  PWMGR_MANAGE_SORTED,
  PWMGR_MANAGE_VISIBILITY_TOGGLED,
  PWMGR_NUM_PASSWORDS_PER_HOSTNAME,
  PWMGR_NUM_SAVED_PASSWORDS,
  PWMGR_NUM_HTTPAUTH_PASSWORDS,
  PWMGR_PASSWORD_INPUT_IN_FORM,
  PWMGR_PROMPT_REMEMBER_ACTION,
  PWMGR_PROMPT_UPDATE_ACTION,
  PWMGR_SAVING_ENABLED,
  PWMGR_USERNAME_PRESENT,
  FENNEC_SYNC11_MIGRATION_SENTINELS_SEEN,
  FENNEC_SYNC11_MIGRATIONS_FAILED,
  FENNEC_SYNC11_MIGRATIONS_SUCCEEDED,
  FENNEC_SYNC11_MIGRATION_NOTIFICATIONS_OFFERED,
  FENNEC_SYNC11_MIGRATIONS_COMPLETED,
  FENNEC_SYNC_NUMBER_OF_SYNCS_STARTED,
  FENNEC_SYNC_NUMBER_OF_SYNCS_COMPLETED,
  FENNEC_SYNC_NUMBER_OF_SYNCS_FAILED,
  FENNEC_SYNC_NUMBER_OF_SYNCS_FAILED_BACKOFF,
  SLOW_SCRIPT_NOTICE_COUNT,
  SLOW_SCRIPT_PAGE_COUNT,
  SLOW_SCRIPT_NOTIFY_DELAY,
  PLUGIN_HANG_NOTICE_COUNT,
  SERVICE_WORKER_SPAWN_ATTEMPTS,
  SERVICE_WORKER_WAS_SPAWNED,
  SERVICE_WORKER_REGISTRATIONS,
  SERVICE_WORKER_CONTROLLED_DOCUMENTS,
  SERVICE_WORKER_UPDATED,
  SERVICE_WORKER_LIFE_TIME,
  GRAPHICS_SANITY_TEST,
  READER_MODE_PARSE_RESULT,
  READER_MODE_DOWNLOAD_RESULT,
  FENNEC_LOAD_SAVED_PAGE,
  PERMISSIONS_SQL_CORRUPTED,
  DEFECTIVE_PERMISSIONS_SQL_REMOVED,
  FENNEC_TABQUEUE_QUEUESIZE,
  FENNEC_CUSTOM_HOMEPAGE,
  GRAPHICS_DRIVER_STARTUP_TEST,
  GRAPHICS_SANITY_TEST_OS_SNAPSHOT,
  DEVTOOLS_HUD_JANK,
  DEVTOOLS_HUD_REFLOW_DURATION,
  DEVTOOLS_HUD_REFLOWS,
  DEVTOOLS_HUD_SECURITY_CATEGORY,
  DEVTOOLS_HUD_ERRORS,
  DEVTOOLS_HUD_WARNINGS,
  DEVTOOLS_HUD_USS,
  DEVTOOLS_HUD_APP_STARTUP_TIME_CONTENTINTERACTIVE,
  DEVTOOLS_HUD_APP_STARTUP_TIME_NAVIGATIONINTERACTIVE,
  DEVTOOLS_HUD_APP_STARTUP_TIME_NAVIGATIONLOADED,
  DEVTOOLS_HUD_APP_STARTUP_TIME_VISUALLYLOADED,
  DEVTOOLS_HUD_APP_STARTUP_TIME_MEDIAENUMERATED,
  DEVTOOLS_HUD_APP_STARTUP_TIME_FULLYLOADED,
  DEVTOOLS_HUD_APP_STARTUP_TIME_SCANEND,
  DEVTOOLS_HUD_APP_MEMORY_CONTENTINTERACTIVE_V2,
  DEVTOOLS_HUD_APP_MEMORY_NAVIGATIONINTERACTIVE_V2,
  DEVTOOLS_HUD_APP_MEMORY_NAVIGATIONLOADED_V2,
  DEVTOOLS_HUD_APP_MEMORY_VISUALLYLOADED_V2,
  DEVTOOLS_HUD_APP_MEMORY_MEDIAENUMERATED_V2,
  DEVTOOLS_HUD_APP_MEMORY_FULLYLOADED_V2,
  DEVTOOLS_HUD_APP_MEMORY_SCANEND_V2,
  DEVTOOLS_MEMORY_TAKE_SNAPSHOT_COUNT,
  DEVTOOLS_MEMORY_IMPORT_SNAPSHOT_COUNT,
  DEVTOOLS_MEMORY_EXPORT_SNAPSHOT_COUNT,
  DEVTOOLS_MEMORY_FILTER_CENSUS,
  DEVTOOLS_MEMORY_DIFF_CENSUS,
  DEVTOOLS_MEMORY_INVERTED_CENSUS,
  DEVTOOLS_MEMORY_BREAKDOWN_CENSUS_COUNT,
  DEVTOOLS_MEMORY_DOMINATOR_TREE_COUNT,
  DEVTOOLS_MEMORY_BREAKDOWN_DOMINATOR_TREE_COUNT,
  GRAPHICS_SANITY_TEST_REASON,
  TRANSLATION_OPPORTUNITIES,
  TRANSLATION_OPPORTUNITIES_BY_LANGUAGE,
  TRANSLATED_PAGES,
  TRANSLATED_PAGES_BY_LANGUAGE,
  TRANSLATED_CHARACTERS,
  DENIED_TRANSLATION_OFFERS,
  AUTO_REJECTED_TRANSLATION_OFFERS,
  REQUESTS_OF_ORIGINAL_CONTENT,
  CHANGES_OF_TARGET_LANGUAGE,
  CHANGES_OF_DETECTED_LANGUAGE,
  SHOULD_TRANSLATION_UI_APPEAR,
  SHOULD_AUTO_DETECT_LANGUAGE,
  PERMISSIONS_REMIGRATION_COMPARISON,
  PERMISSIONS_MIGRATION_7_ERROR,
  LOOP_ICE_FINAL_CONNECTION_STATE,
  LOOP_ICE_ON_TIME_TRICKLE_ARRIVAL_TIME,
  LOOP_ICE_LATE_TRICKLE_ARRIVAL_TIME,
  LOOP_ICE_SUCCESS_TIME,
  LOOP_ICE_FAILURE_TIME,
  LOOP_ICE_SUCCESS_RATE,
  LOOP_VIDEO_QUALITY_INBOUND_BANDWIDTH_KBITS,
  LOOP_AUDIO_QUALITY_INBOUND_BANDWIDTH_KBITS,
  LOOP_VIDEO_QUALITY_OUTBOUND_BANDWIDTH_KBITS,
  LOOP_AUDIO_QUALITY_OUTBOUND_BANDWIDTH_KBITS,
  LOOP_VIDEO_ERROR_RECOVERY_MS,
  LOOP_VIDEO_RECOVERY_BEFORE_ERROR_PER_MIN,
  LOOP_VIDEO_RECOVERY_AFTER_ERROR_PER_MIN,
  LOOP_VIDEO_DECODE_ERROR_TIME_PERMILLE,
  LOOP_VIDEO_QUALITY_OUTBOUND_RTT,
  LOOP_AUDIO_QUALITY_OUTBOUND_RTT,
  LOOP_VIDEO_ENCODER_BITRATE_AVG_PER_CALL_KBPS,
  LOOP_VIDEO_ENCODER_BITRATE_STD_DEV_PER_CALL_KBPS,
  LOOP_VIDEO_ENCODER_FRAMERATE_AVG_PER_CALL,
  LOOP_VIDEO_ENCODER_FRAMERATE_10X_STD_DEV_PER_CALL,
  LOOP_VIDEO_ENCODER_DROPPED_FRAMES_PER_CALL_FPM,
  LOOP_VIDEO_DECODER_BITRATE_AVG_PER_CALL_KBPS,
  LOOP_VIDEO_DECODER_BITRATE_STD_DEV_PER_CALL_KBPS,
  LOOP_VIDEO_DECODER_FRAMERATE_AVG_PER_CALL,
  LOOP_VIDEO_DECODER_FRAMERATE_10X_STD_DEV_PER_CALL,
  LOOP_VIDEO_DECODER_DISCARDED_PACKETS_PER_CALL_PPM,
  LOOP_CALL_DURATION,
  LOOP_ICE_ADD_CANDIDATE_ERRORS_GIVEN_SUCCESS,
  LOOP_ICE_ADD_CANDIDATE_ERRORS_GIVEN_FAILURE,
  LOOP_GET_USER_MEDIA_TYPE,
  LOOP_RENEGOTIATIONS,
  LOOP_MAX_VIDEO_SEND_TRACK,
  LOOP_MAX_VIDEO_RECEIVE_TRACK,
  LOOP_MAX_AUDIO_SEND_TRACK,
  LOOP_MAX_AUDIO_RECEIVE_TRACK,
  LOOP_DATACHANNEL_NEGOTIATED,
  LOOP_CALL_TYPE,
  PERF_MONITORING_TEST_CPU_RESCHEDULING_PROPORTION_MOVED,
  PERF_MONITORING_SLOW_ADDON_JANK_US,
  PERF_MONITORING_SLOW_ADDON_CPOW_US,
  VIDEO_EME_REQUEST_SUCCESS_LATENCY_MS,
  VIDEO_EME_REQUEST_FAILURE_LATENCY_MS,
  FXA_CONFIGURED,
  WEAVE_DEVICE_COUNT_DESKTOP,
  WEAVE_DEVICE_COUNT_MOBILE,
  WEAVE_ENGINE_SYNC_ERRORS,
  CONTENT_DOCUMENTS_DESTROYED,
  TOP_LEVEL_CONTENT_DOCUMENTS_DESTROYED,
  PUSH_API_USED,
  PUSH_API_PERMISSION_REQUESTED,
  PUSH_API_PERMISSION_DENIED,
  PUSH_API_PERMISSION_GRANTED,
  PUSH_API_SUBSCRIBE_ATTEMPT,
  PUSH_API_SUBSCRIBE_FAILED,
  PUSH_API_SUBSCRIBE_SUCCEEDED,
  PUSH_API_UNSUBSCRIBE_ATTEMPT,
  PUSH_API_UNSUBSCRIBE_FAILED,
  PUSH_API_UNSUBSCRIBE_SUCCEEDED,
  PUSH_API_SUBSCRIBE_WS_TIME,
  PUSH_API_SUBSCRIBE_HTTP2_TIME,
  PUSH_API_QUOTA_EXPIRATION_TIME,
  PUSH_API_QUOTA_RESET_TO,
  PUSH_API_NOTIFICATION_RECEIVED,
  PUSH_API_NOTIFICATION_RECEIVED_BUT_DID_NOT_NOTIFY,
  PUSH_API_NOTIFY,
  PUSH_API_NOTIFY_REGISTRATION_LOST,
  D3D11_SYNC_HANDLE_FAILURE,
  GFX_CONTENT_FAILED_TO_ACQUIRE_DEVICE,
  GFX_CRASH,
  PLUGIN_ACTIVATION_COUNT,
  SCROLL_INPUT_METHODS,
  WEB_NOTIFICATION_CLICKED,
  WEB_NOTIFICATION_MENU,
  WEB_NOTIFICATION_SHOWN,
  WEBFONT_DOWNLOAD_TIME,
  WEBFONT_DOWNLOAD_TIME_AFTER_START,
  WEBFONT_FONTTYPE,
  WEBFONT_SRCTYPE,
  WEBFONT_PER_PAGE,
  WEBFONT_SIZE_PER_PAGE,
  WEBFONT_SIZE,
  WEBFONT_COMPRESSION_WOFF,
  WEBFONT_COMPRESSION_WOFF2,
  WEBRTC_ICE_CHECKING_RATE,
  ALERTS_SERVICE_DND_ENABLED,
  ALERTS_SERVICE_DND_SUPPORTED_FLAG,
  WEB_NOTIFICATION_EXCEPTIONS_OPENED,
  WEB_NOTIFICATION_PERMISSIONS,
  WEB_NOTIFICATION_PERMISSION_REMOVED,
  WEB_NOTIFICATION_SENDERS,
  YOUTUBE_REWRITABLE_EMBED_SEEN,
  YOUTUBE_NONREWRITABLE_EMBED_SEEN,
  PLUGIN_DRAWING_MODEL,
  WEB_NOTIFICATION_REQUEST_PERMISSION_CALLBACK,
  VIDEO_FASTSEEK_USED,
  VIDEO_DROPPED_FRAMES_PROPORTION,
  TAB_SWITCH_CACHE_POSITION,
  REMOTE_JAR_PROTOCOL_USED,
  MEDIA_DECODER_BACKEND_USED,
  PLUGIN_BLOCKED_FOR_STABILITY,
  PLUGIN_TINY_CONTENT,
  IPC_MESSAGE_SIZE,
  IPC_REPLY_SIZE,
  MESSAGE_MANAGER_MESSAGE_SIZE2,
  REJECTED_MESSAGE_MANAGER_MESSAGE,
  SANDBOX_BROKER_INITIALIZED,
#if defined(XP_LINUX)
  SANDBOX_HAS_SECCOMP_BPF,
#endif
#if defined(XP_LINUX)
  SANDBOX_HAS_SECCOMP_TSYNC,
#endif
#if defined(XP_LINUX)
  SANDBOX_HAS_USER_NAMESPACES,
#endif
#if defined(XP_LINUX)
  SANDBOX_HAS_USER_NAMESPACES_PRIVILEGED,
#endif
#if defined(XP_LINUX)
  SANDBOX_MEDIA_ENABLED,
#endif
#if defined(XP_LINUX)
  SANDBOX_CONTENT_ENABLED,
#endif
  SYNC_WORKER_OPERATION,
  SUBPROCESS_KILL_HARD,
  FX_CONTENT_CRASH_DUMP_UNAVAILABLE,
  FX_CONTENT_CRASH_PRESENTED,
  FX_CONTENT_CRASH_NOT_SUBMITTED,
  ABOUTCRASHES_OPENED_COUNT,
  D3D9_COMPOSITING_FAILURE_ID,
  D3D11_COMPOSITING_FAILURE_ID,
  OPENGL_COMPOSITING_FAILURE_ID,
  XHR_IN_WORKER,
  WEBVTT_TRACK_KINDS,
  WEBVTT_USED_VTT_CUES,
  BLINK_FILESYSTEM_USED,
  WEBKIT_DIRECTORY_USED,
  CONTAINER_USED,
  UNIQUE_CONTAINERS_OPENED,
  TOTAL_CONTAINERS_OPENED,
#if defined(ANDROID)
  FENNEC_SESSIONSTORE_DAMAGED_SESSION_FILE,
#endif
  HistogramFirstUseCounter,
  HistogramDUMMY1 = HistogramFirstUseCounter - 1,
  USE_COUNTER2_SVGSVGELEMENT_GETELEMENTBYID_DOCUMENT,
  USE_COUNTER2_SVGSVGELEMENT_GETELEMENTBYID_PAGE,
  USE_COUNTER2_SVGSVGELEMENT_CURRENTSCALE_getter_DOCUMENT,
  USE_COUNTER2_SVGSVGELEMENT_CURRENTSCALE_getter_PAGE,
  USE_COUNTER2_SVGSVGELEMENT_CURRENTSCALE_setter_DOCUMENT,
  USE_COUNTER2_SVGSVGELEMENT_CURRENTSCALE_setter_PAGE,
  USE_COUNTER2_PROPERTY_FILL_DOCUMENT,
  USE_COUNTER2_PROPERTY_FILL_PAGE,
  USE_COUNTER2_PROPERTY_FILLOPACITY_DOCUMENT,
  USE_COUNTER2_PROPERTY_FILLOPACITY_PAGE,
  USE_COUNTER2_PUSHMANAGER_SUBSCRIBE_DOCUMENT,
  USE_COUNTER2_PUSHMANAGER_SUBSCRIBE_PAGE,
  USE_COUNTER2_PUSHSUBSCRIPTION_UNSUBSCRIBE_DOCUMENT,
  USE_COUNTER2_PUSHSUBSCRIPTION_UNSUBSCRIBE_PAGE,
  USE_COUNTER2_WINDOW_SIDEBAR_getter_DOCUMENT,
  USE_COUNTER2_WINDOW_SIDEBAR_getter_PAGE,
  USE_COUNTER2_WINDOW_SIDEBAR_setter_DOCUMENT,
  USE_COUNTER2_WINDOW_SIDEBAR_setter_PAGE,
  USE_COUNTER2_EXTERNAL_ADDSEARCHENGINE_DOCUMENT,
  USE_COUNTER2_EXTERNAL_ADDSEARCHENGINE_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_SWAPCACHE_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_SWAPCACHE_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_UPDATE_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_UPDATE_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_STATUS_getter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_STATUS_getter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_STATUS_setter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_STATUS_setter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONCHECKING_getter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONCHECKING_getter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONCHECKING_setter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONCHECKING_setter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONERROR_getter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONERROR_getter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONERROR_setter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONERROR_setter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONNOUPDATE_getter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONNOUPDATE_getter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONNOUPDATE_setter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONNOUPDATE_setter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONDOWNLOADING_getter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONDOWNLOADING_getter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONDOWNLOADING_setter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONDOWNLOADING_setter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONPROGRESS_getter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONPROGRESS_getter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONPROGRESS_setter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONPROGRESS_setter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONUPDATEREADY_getter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONUPDATEREADY_getter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONUPDATEREADY_setter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONUPDATEREADY_setter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONCACHED_getter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONCACHED_getter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONCACHED_setter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONCACHED_setter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONOBSOLETE_getter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONOBSOLETE_getter_PAGE,
  USE_COUNTER2_OFFLINERESOURCELIST_ONOBSOLETE_setter_DOCUMENT,
  USE_COUNTER2_OFFLINERESOURCELIST_ONOBSOLETE_setter_PAGE,
  USE_COUNTER2_DEPRECATED_GetAttributeNode_DOCUMENT,
  USE_COUNTER2_DEPRECATED_GetAttributeNode_PAGE,
  USE_COUNTER2_DEPRECATED_SetAttributeNode_DOCUMENT,
  USE_COUNTER2_DEPRECATED_SetAttributeNode_PAGE,
  USE_COUNTER2_DEPRECATED_GetAttributeNodeNS_DOCUMENT,
  USE_COUNTER2_DEPRECATED_GetAttributeNodeNS_PAGE,
  USE_COUNTER2_DEPRECATED_SetAttributeNodeNS_DOCUMENT,
  USE_COUNTER2_DEPRECATED_SetAttributeNodeNS_PAGE,
  USE_COUNTER2_DEPRECATED_RemoveAttributeNode_DOCUMENT,
  USE_COUNTER2_DEPRECATED_RemoveAttributeNode_PAGE,
  USE_COUNTER2_DEPRECATED_CreateAttribute_DOCUMENT,
  USE_COUNTER2_DEPRECATED_CreateAttribute_PAGE,
  USE_COUNTER2_DEPRECATED_CreateAttributeNS_DOCUMENT,
  USE_COUNTER2_DEPRECATED_CreateAttributeNS_PAGE,
  USE_COUNTER2_DEPRECATED_NodeValue_DOCUMENT,
  USE_COUNTER2_DEPRECATED_NodeValue_PAGE,
  USE_COUNTER2_DEPRECATED_TextContent_DOCUMENT,
  USE_COUNTER2_DEPRECATED_TextContent_PAGE,
  USE_COUNTER2_DEPRECATED_EnablePrivilege_DOCUMENT,
  USE_COUNTER2_DEPRECATED_EnablePrivilege_PAGE,
  USE_COUNTER2_DEPRECATED_DOMExceptionCode_DOCUMENT,
  USE_COUNTER2_DEPRECATED_DOMExceptionCode_PAGE,
  USE_COUNTER2_DEPRECATED_NoExposedProps_DOCUMENT,
  USE_COUNTER2_DEPRECATED_NoExposedProps_PAGE,
  USE_COUNTER2_DEPRECATED_MutationEvent_DOCUMENT,
  USE_COUNTER2_DEPRECATED_MutationEvent_PAGE,
  USE_COUNTER2_DEPRECATED_Components_DOCUMENT,
  USE_COUNTER2_DEPRECATED_Components_PAGE,
  USE_COUNTER2_DEPRECATED_PrefixedVisibilityAPI_DOCUMENT,
  USE_COUNTER2_DEPRECATED_PrefixedVisibilityAPI_PAGE,
  USE_COUNTER2_DEPRECATED_NodeIteratorDetach_DOCUMENT,
  USE_COUNTER2_DEPRECATED_NodeIteratorDetach_PAGE,
  USE_COUNTER2_DEPRECATED_LenientThis_DOCUMENT,
  USE_COUNTER2_DEPRECATED_LenientThis_PAGE,
  USE_COUNTER2_DEPRECATED_GetPreventDefault_DOCUMENT,
  USE_COUNTER2_DEPRECATED_GetPreventDefault_PAGE,
  USE_COUNTER2_DEPRECATED_GetSetUserData_DOCUMENT,
  USE_COUNTER2_DEPRECATED_GetSetUserData_PAGE,
  USE_COUNTER2_DEPRECATED_MozGetAsFile_DOCUMENT,
  USE_COUNTER2_DEPRECATED_MozGetAsFile_PAGE,
  USE_COUNTER2_DEPRECATED_UseOfCaptureEvents_DOCUMENT,
  USE_COUNTER2_DEPRECATED_UseOfCaptureEvents_PAGE,
  USE_COUNTER2_DEPRECATED_UseOfReleaseEvents_DOCUMENT,
  USE_COUNTER2_DEPRECATED_UseOfReleaseEvents_PAGE,
  USE_COUNTER2_DEPRECATED_UseOfDOM3LoadMethod_DOCUMENT,
  USE_COUNTER2_DEPRECATED_UseOfDOM3LoadMethod_PAGE,
  USE_COUNTER2_DEPRECATED_ShowModalDialog_DOCUMENT,
  USE_COUNTER2_DEPRECATED_ShowModalDialog_PAGE,
  USE_COUNTER2_DEPRECATED_Window_Content_DOCUMENT,
  USE_COUNTER2_DEPRECATED_Window_Content_PAGE,
  USE_COUNTER2_DEPRECATED_SyncXMLHttpRequest_DOCUMENT,
  USE_COUNTER2_DEPRECATED_SyncXMLHttpRequest_PAGE,
  USE_COUNTER2_DEPRECATED_DataContainerEvent_DOCUMENT,
  USE_COUNTER2_DEPRECATED_DataContainerEvent_PAGE,
  USE_COUNTER2_DEPRECATED_Window_Controllers_DOCUMENT,
  USE_COUNTER2_DEPRECATED_Window_Controllers_PAGE,
  USE_COUNTER2_DEPRECATED_ImportXULIntoContent_DOCUMENT,
  USE_COUNTER2_DEPRECATED_ImportXULIntoContent_PAGE,
  USE_COUNTER2_DEPRECATED_PannerNodeDoppler_DOCUMENT,
  USE_COUNTER2_DEPRECATED_PannerNodeDoppler_PAGE,
  USE_COUNTER2_DEPRECATED_NavigatorGetUserMedia_DOCUMENT,
  USE_COUNTER2_DEPRECATED_NavigatorGetUserMedia_PAGE,
  USE_COUNTER2_DEPRECATED_WebrtcDeprecatedPrefix_DOCUMENT,
  USE_COUNTER2_DEPRECATED_WebrtcDeprecatedPrefix_PAGE,
  USE_COUNTER2_DEPRECATED_RTCPeerConnectionGetStreams_DOCUMENT,
  USE_COUNTER2_DEPRECATED_RTCPeerConnectionGetStreams_PAGE,
  USE_COUNTER2_DEPRECATED_AppCache_DOCUMENT,
  USE_COUNTER2_DEPRECATED_AppCache_PAGE,
  USE_COUNTER2_DEPRECATED_PrefixedFullscreenAPI_DOCUMENT,
  USE_COUNTER2_DEPRECATED_PrefixedFullscreenAPI_PAGE,
  USE_COUNTER2_DEPRECATED_LenientSetter_DOCUMENT,
  USE_COUNTER2_DEPRECATED_LenientSetter_PAGE,
  USE_COUNTER2_DEPRECATED_FileLastModifiedDate_DOCUMENT,
  USE_COUNTER2_DEPRECATED_FileLastModifiedDate_PAGE,
  HistogramDUMMY2,
  HistogramLastUseCounter = HistogramDUMMY2 - 1,
  HistogramCount,
  HistogramUseCounterCount = HistogramLastUseCounter - HistogramFirstUseCounter + 1
};

enum class LABELS_TELEMETRY_TEST_CATEGORICAL : uint32_t {
  CommonLabel,
  Label2,
  Label3
};

enum class LABELS_TELEMETRY_TEST_CATEGORICAL_OPTOUT : uint32_t {
  CommonLabel,
  Label4,
  Label5,
  Label6
};

template<class T> struct IsCategoricalLabelEnum : FalseType {};
template<> struct IsCategoricalLabelEnum<LABELS_TELEMETRY_TEST_CATEGORICAL> : TrueType {};
template<> struct IsCategoricalLabelEnum<LABELS_TELEMETRY_TEST_CATEGORICAL_OPTOUT> : TrueType {};

template<class T> struct CategoricalLabelId {};
template<> struct CategoricalLabelId<LABELS_TELEMETRY_TEST_CATEGORICAL> : IntegralConstant<uint32_t, TELEMETRY_TEST_CATEGORICAL> {};
template<> struct CategoricalLabelId<LABELS_TELEMETRY_TEST_CATEGORICAL_OPTOUT> : IntegralConstant<uint32_t, TELEMETRY_TEST_CATEGORICAL_OPTOUT> {};

} // namespace mozilla
} // namespace Telemetry
#endif // mozilla_TelemetryHistogramEnums_h
