/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(99e81922-7318-4431-b3aa-78b3cb4119bb)]
interface nsICryptoFIPSInfo : nsISupports
{
    readonly attribute boolean isFIPSModeActive;
};

%{C++
#define NS_CRYPTO_FIPSINFO_SERVICE_CONTRACTID "@mozilla.org/crypto/fips-info-service;1"
%}
