#ifndef mozilla_dom_RegisterBindings_h
#define mozilla_dom_RegisterBindings_h


namespace mozilla {
namespace dom {
void
RegisterWebIDLGlobalNames();

} // namespace dom
} // namespace mozilla


#endif // mozilla_dom_RegisterBindings_h
