#define MOZ_SOURCE_STAMP 6efc0964ec62bc4abfdc4cb1dc7cc461c3238634
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/6efc0964ec62bc4abfdc4cb1dc7cc461c3238634
